import os
from datetime import datetime, timedelta
from aiogram import Bot, Dispatcher, types
from aiogram.filters import Command
from aiogram.fsm.context import FSMContext
from aiogram.fsm.state import State, StatesGroup
from aiogram.fsm.storage.memory import MemoryStorage
from aiogram.types import InlineKeyboardButton, InlineKeyboardMarkup, ReplyKeyboardMarkup, KeyboardButton
import logging
import json
from pathlib import Path
from dotenv import load_dotenv
import regex

from AdminPanelUNI import register_admin_handlers, get_admin_keyboard, banned_words, bot_texts

import asyncio

# بارگذاری متغیرهای محیطی
load_dotenv()

# دریافت توکن از محیط
TOKEN = os.getenv('TELEGRAM_BOT_TOKEN')

if not TOKEN:
    raise ValueError("توکن ربات در فایل .env یافت نشد!")


DATA_DIR = Path("data")
DATA_DIR2 = Path("Initialdata")
DATA_DIR.mkdir(exist_ok=True)  # این خط پوشه را ایجاد می‌کند اگر وجود نداشته باشد

# تنظیمات پایه
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    filename = DATA_DIR / 'bot.log'
)
logger = logging.getLogger(__name__)

# تنظیمات ربات
ADMIN_ID = 7710387966  # آیدی عددی ادمین اصلی
ADMIN_IDM = 7710387966  # آیدی عددی ادمین اصلی
CHANNEL_ID = "@UniMarket_ir"  # آیدی کانال
CHANNEL_IDM = "UniMarket_ir"  # آیدی کانال بدون @
SUPPORT_USERNAME = "ShUAdmin"  # یوزرنیم پشتیبانی
SUBMISSION_INTERVAL = 30  # حداقل فاصله ارسال به ثانیه
INVITE_REWARD = 5  # امتیاز هر دعوت موفق
time_description = "هفته"  # توضیح بازه زمانی
T_day_of_week = 'fri'
T_hour = 22
T_minute = 00
T_second = 00
# تنظیمات محدودیت درخواست‌ها
REQUEST_LIMIT = 7  # حداکثر تعداد درخواست در بازه زمانی
REQUEST_WINDOW = 20  # بازه زمانی به ثانیه (30 ثانیه)
MAX_FILE_SIZE = 50 * 1024 * 1024  # 50 مگابایت
DAILY_SUBMISSION_LIMIT = 3  # حداکثر تعداد آگهی در روز
DAILY_SUBMISSION_REWARD = 5  # امتیاز برای ارسال روزانه
SystemRun = "local"

# لیست دسته‌بندی‌ها را در بخش تنظیمات ربات اضافه کنید:
CATEGORIES = [
    "🏠 هم‌خونه",
    "📚 کتاب و نوشت‌افزار",
    "📱 کالای دیجیتال",
    "🍽️ خانه و آشپزخانه",
    "💄 آرایشی و بهداشتی",
    "🎓 خدمات دانشجویی",
    "👕 وسایل شخصی",
    "🎮 سرگرمی و فراغت",
    "💼 استخدام و کاریابی",
    "📌 سایر"
]

user_request_timestamps = {}  # برای ذخیره زمان‌های درخواست کاربران
user_violations = {}  # برای ذخیره تعداد تخلفات کاربران
user_blocks = {}  # برای ذخیره زمان پایان مسدودیت کاربران

USER_MEMORIES_FILE = DATA_DIR / "user_memories.json"  # فایل ذخیره لینک آگهی‌های کاربران
def load_user_memories():
    """بارگذاری آگهی‌های کاربران از فایل"""
    try:
        if USER_MEMORIES_FILE.exists():
            with open(USER_MEMORIES_FILE, 'r', encoding='utf-8') as f:
                data = json.load(f)
                # تبدیل وضعیت‌های قدیمی به جدید
                for user_id, memories in data.items():
                    for memory in memories:
                        if 'status' not in memory:
                            # اگر پیام در کانال وجود ندارد، آن را به عنوان حذف شده علامت بزنید
                            if 'message_id' in memory:
                                try:
                                    message = bot.get_chat_message(CHANNEL_ID, memory['message_id'])
                                    memory['status'] = 'active'
                                except:
                                    memory['status'] = 'deleted'
                            else:
                                memory['status'] = 'active'
                return data
    except Exception as e:
        logger.error(f"Error loading user memories: {e}")
    return {}

def save_user_memories(data):
    """ذخیره آگهی‌های کاربران در فایل"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(USER_MEMORIES_FILE, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving user memories: {e}")


ADMINS_FILE = DATA_DIR / "admins.json"  # فایل ذخیره ادمین‌ها
BLOCKED_USERS_FILE = DATA_DIR / "blocked_users.json"  # فایل ذخیره کاربران بلاک شده
# فایل‌های ذخیره‌سازی
USER_DATA_FILE = DATA_DIR / "user_data.json"
USER_SCORES_FILE = DATA_DIR / "user_scores.json"
RATING_VOTES_FILE = DATA_DIR / "rating_votes.json"
LAST_SUBMISSIONS_FILE = DATA_DIR / "last_submissions.json"
# در بخش تعریف متغیرهای global
channel_posts_store = {}  # برای ذخیره اطلاعات پست‌های کانال
CHANNEL_POSTS_FILE = DATA_DIR / "channel_posts.json"  # فایل ذخیره‌سازی
admin_messages_tracking = {}  # {media_unique_id: {admin_id: message_id}}

# در بخش تعریف متغیرهای global
ADMINS_DATA_FILE = DATA_DIR / "admins_data.json"  # فایل ذخیره اطلاعات ادمین‌ها

RULES_FILE = DATA_DIR / "accepted_rules.json"  # فایل ذخیره وضعیت پذیرش قوانین

USER_BLOCKS_FILE = DATA_DIR / "user_blocks.json"  # فایل ذخیره زمان‌های مسدودیت کاربران

def load_user_blocks():
    """بارگذاری زمان‌های مسدودیت از فایل"""
    try:
        if USER_BLOCKS_FILE.exists():
            with open(USER_BLOCKS_FILE, 'r', encoding='utf-8') as f:
                data = json.load(f)
                # تبدیل رشته‌های تاریخ به شیء datetime
                return {
                    int(user_id): datetime.fromisoformat(block_time)
                    for user_id, block_time in data.items()
                }
    except Exception as e:
        logger.error(f"Error loading user blocks: {e}")
    return {}

def save_user_blocks(data):
    """ذخیره زمان‌های مسدودیت در فایل"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(USER_BLOCKS_FILE, 'w', encoding='utf-8') as f:
            # تبدیل شیء datetime به رشته قابل ذخیره
            json.dump(
                {user_id: block_time.isoformat() for user_id, block_time in data.items()},
                f,
                ensure_ascii=False,
                indent=2
            )
    except Exception as e:
        logger.error(f"Error saving user blocks: {e}")

# در بخش تعریف فایل‌های داده (بالای کد)
SUPPORT_REQUESTS_FILE = DATA_DIR / "support_requests.json"  # فایل ذخیره درخواست‌های پشتیبانی

def load_support_requests():
    """بارگذاری تعداد درخواست‌های پشتیبانی از فایل"""
    try:
        if SUPPORT_REQUESTS_FILE.exists():
            with open(SUPPORT_REQUESTS_FILE, 'r', encoding='utf-8') as f:
                return json.load(f)
    except Exception as e:
        logger.error(f"Error loading support requests: {e}")
    return {}

def save_support_requests(data):
    """ذخیره تعداد درخواست‌های پشتیبانی در فایل"""
    try:
        DATA_DIR.mkdir(exist_ok=True)  # اصلاح این خط
        with open(SUPPORT_REQUESTS_FILE, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving support requests: {e}")

support_requests = load_support_requests()  # فایل ذخیره درخواست‌های پشتیبانی

report_messages = {}  # {report_id: {admin_id: message_id}}
REPORTS_FILE = DATA_DIR / "user_reports.json"  # فایل ذخیره گزارش‌ها

def load_reports_data():
    """بارگذاری گزارش‌های کاربران از فایل"""
    try:
        if REPORTS_FILE.exists():
            with open(REPORTS_FILE, 'r', encoding='utf-8') as f:
                data = json.load(f)
                if isinstance(data, dict) and not any('status' in item for item in data.values()):
                    new_data = {}
                    for user_id, reports in data.items():
                        new_data[user_id] = {}
                        for report_id, report_data in reports.items():
                            new_data[user_id][report_id] = {
                                **report_data,
                                'status': 'pending'  # گزارش‌های قدیمی را به عنوان pending در نظر بگیرید
                            }
                    return new_data
                return data
    except Exception as e:
        logger.error(f"Error loading reports data: {e}")
    return {}

def save_reports_data(data):
    """ذخیره گزارش‌های کاربران در فایل"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(REPORTS_FILE, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving reports data: {e}")


def load_accepted_rules():
    """بارگذاری وضعیت پذیرش قوانین"""
    try:
        if RULES_FILE.exists():
            with open(RULES_FILE, 'r', encoding='utf-8') as f:
                return json.load(f)
    except Exception as e:
        logger.error(f"Error loading accepted rules: {e}")
    return {}

def save_accepted_rules(data):
    """ذخیره وضعیت پذیرش قوانین"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(RULES_FILE, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving accepted rules: {e}")

def has_accepted_rules(user_id: int) -> bool:
    """بررسی آیا کاربر قوانین را پذیرفته است"""
    accepted_rules = load_accepted_rules()
    return str(user_id) in accepted_rules



async def get_user_info(user_id: int):
    """دریافت اطلاعات کاربر از تلگرام"""
    try:
        user = await bot.get_chat(user_id)
        return {
            'id': user.id,
            'first_name': user.first_name,
            'last_name': getattr(user, 'last_name', ''),
            'username': getattr(user, 'username', ''),
            'date': datetime.now().isoformat()
        }
    except Exception as e:
        logger.error(f"Error getting user info for {user_id}: {e}")
        return None

def load_admins_data():
    """بارگذاری اطلاعات کامل ادمین‌ها"""
    try:
        if ADMINS_DATA_FILE.exists():
            with open(ADMINS_DATA_FILE, 'r', encoding='utf-8') as f:
                return json.load(f)
    except Exception as e:
        logger.error(f"Error loading admins data: {e}")
    return {}

def save_admins_data(admins_data):
    """ذخیره اطلاعات کامل ادمین‌ها"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(ADMINS_DATA_FILE, 'w', encoding='utf-8') as f:
            json.dump(admins_data, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving admins data: {e}")

def save_data_to_file(data, file_path):
    """ذخیره داده‌ها در فایل JSON با پشتیبانی از datetime"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        
        def json_serializer(obj):
            if isinstance(obj, datetime):
                return obj.isoformat()
            raise TypeError(f"Type {type(obj)} not serializable")
            
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2, default=json_serializer)
    except Exception as e:
        logger.error(f"Error saving data to {file_path}: {e}")

def load_data_from_file(file_path, default=None):
    try:
        if file_path.exists():
            with open(file_path, 'r', encoding='utf-8') as f:
                return json.load(f)
    except Exception as e:
        logger.error(f"Error loading data from {file_path}: {e}")
    return default if default is not None else {}

def convert_keys_to_int(d):
    if not d:
        return {}
        
    if isinstance(d, list):
        return [convert_keys_to_int(item) for item in d]
        
    if isinstance(d, dict):
        new_dict = {}
        for k, v in d.items():
            # Only convert keys that are numeric strings
            if isinstance(k, str) and k.isdigit():
                new_key = int(k)
            else:
                new_key = k
            new_dict[new_key] = convert_keys_to_int(v)
        return new_dict
        
    return d

def check_daily_submission_limit(user_id: int) -> bool:
    """بررسی آیا کاربر به حد مجاز ارسال روزانه رسیده است (بدون اعطای پاداش)"""
    try:
        today = datetime.now().date()
        daily_count = 0

        if user_id in user_data_store:
            for memory in user_data_store[user_id]:
                if isinstance(memory, dict):
                    submission_time = memory.get('submission_time')
                    if submission_time:
                        if isinstance(submission_time, str):
                            submission_time = datetime.fromisoformat(submission_time)
                        if submission_time.date() == today:
                            daily_count += 1
        
        return int((daily_count-1)/2+1) >= int(DAILY_SUBMISSION_LIMIT+1)
        
    except Exception as e:
        logger.error(f"Error in daily limit check: {e}")
        return False
        

def grant_daily_reward(user_id: int) -> bool:
    """اعطای پاداش روزانه فقط در صورت رسیدن به حد مجاز ارسال روزانه"""
    try:
        today = datetime.now().date()
        today_str = today.isoformat()
        
        # بررسی تعداد آگهی‌های ارسالی کاربر در امروز
        daily_count = 0
        if user_id in user_data_store:
            for memory in user_data_store[user_id]:
                if isinstance(memory, dict):
                    submission_time = memory.get('submission_time')
                    if submission_time:
                        if isinstance(submission_time, str):
                            submission_time = datetime.fromisoformat(submission_time)
                        if submission_time.date() == today:
                            daily_count += 1
        
        # اگر کاربر به حد مجاز نرسیده باشد، پاداش ندهید
        if int((daily_count-1)/2+1) < int(DAILY_SUBMISSION_LIMIT):
            return False
            
        # بررسی آیا کاربر امروز پاداش گرفته یا نه
        if user_id in user_scores and today_str in user_scores[user_id].daily_rewards:
            return False  # کاربر قبلاً پاداش گرفته
            
        # اعطای پاداش
        if user_id not in user_scores:
            user_scores[user_id] = UserScore()
        
        user_scores[user_id].total_score += DAILY_SUBMISSION_REWARD
        user_scores[user_id].daily_rewards[today_str] = True
        save_all_data()  # ذخیره تغییرات
        return True
        
    except Exception as e:
        logger.error(f"Error granting daily reward: {e}")
        return False

from aiogram.fsm.storage.redis import RedisStorage
# ذخیره‌سازی
if SystemRun == "local":
    storage = MemoryStorage()
else:
    storage = RedisStorage.from_url("redis://localhost:6379/0")  # اتصال به Redis محلی


bot = Bot(token=TOKEN)
BOT_USERNAME = None  # این خط را اضافه کنید

dp = Dispatcher(storage=storage)

@dp.callback_query(lambda c: c.data == "accept_rules")
async def accept_rules(callback: types.CallbackQuery):
    """پردازش پذیرش قوانین توسط کاربر"""
    try:
        user_id = callback.from_user.id
        
        # ذخیره وضعیت پذیرش
        accepted_rules = load_accepted_rules()
        accepted_rules[str(user_id)] = {
            'date': datetime.now().isoformat(),
            'username': callback.from_user.username or ''
        }
        save_accepted_rules(accepted_rules)
        
        await callback.message.delete()

        if user_id in admins:
            await callback.message.answer(
                "✅ شما تمام قوانین را پذیرفتید. ✅ \n مجدداً روی دکمه 💠 ارسال آگهی کلیک کنید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await callback.message.answer(
                "✅ شما تمام قوانین را پذیرفتید. ✅ \n مجدداً روی دکمه 💠 ارسال آگهی کلیک کنید.",
                reply_markup=get_main_keyboard()
            )

        await callback.answer()
    except Exception as e:
        logger.error(f"Error in accept_rules: {e}")
        await callback.answer("⚠️ خطا در ثبت پذیرش قوانین! \n با پشتیبانی تماس بگیرید.")

def load_admins():
    """بارگذاری لیست ادمین‌ها از فایل"""
    try:
        if ADMINS_FILE.exists():
            with open(ADMINS_FILE, 'r', encoding='utf-8') as f:
                data = json.load(f)
                return set(map(int, data.get('admins', [])))
    except Exception as e:
        logger.error(f"Error loading admins: {e}")
    return {ADMIN_ID}  # ادمین اصلی به عنوان پیش‌فرض

def save_admins(admins):
    """ذخیره لیست ادمین‌ها در فایل"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(ADMINS_FILE, 'w', encoding='utf-8') as f:
            json.dump({'admins': list(admins)}, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving admins: {e}")

def load_blocked_users():
    """بارگذاری لیست کاربران بلاک شده از فایل"""
    try:
        if BLOCKED_USERS_FILE.exists():
            with open(BLOCKED_USERS_FILE, 'r', encoding='utf-8') as f:
                data = json.load(f)
                return set(map(int, data.get('blocked_users', [])))
    except Exception as e:
        logger.error(f"Error loading blocked users: {e}")
    return set()

def save_blocked_users(blocked_users):
    """ذخیره لیست کاربران بلاک شده در فایل"""
    try:
        DATA_DIR.mkdir(exist_ok=True)
        with open(BLOCKED_USERS_FILE, 'w', encoding='utf-8') as f:
            json.dump({'blocked_users': list(blocked_users)}, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving blocked users: {e}")

def is_user_blocked(user_id: int) -> bool:
    """بررسی آیا کاربر بلاک شده است یا نه"""
    blocked_users = load_blocked_users()  # همیشه از فایل بخواند
    return user_id in blocked_users

def save_report_messages(data):
    """ذخیره گزارش‌های فعال در فایل"""
    try:
        with open(DATA_DIR / "active_reports.json", 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"Error saving report messages: {e}")

def load_report_messages():
    """بارگذاری گزارش‌های فعال از فایل"""
    try:
        if (DATA_DIR / "active_reports.json").exists():
            with open(DATA_DIR / "active_reports.json", 'r', encoding='utf-8') as f:
                return json.load(f)
    except Exception as e:
        logger.error(f"Error loading report messages: {e}")
    return {}

report_messages = load_report_messages()  # بارگذاری از فایل
# متغیرهای جدید
# admins = load_admins()  # مجموعه ادمین‌ها
def get_current_admins():
    """همیشه لیست ادمین‌ها را از فایل می‌خواند"""
    # admins_data = load_admins_data()
    return load_admins()

def get_current_adminsdata():
    """همیشه دیتا ادمین‌ها را از فایل می‌خواند"""
    return load_admins_data()
blocked_users = load_blocked_users()  # مجموعه کاربران بلاک شده

from apscheduler.schedulers.asyncio import AsyncIOScheduler
scheduler = AsyncIOScheduler()

# در بخش ذخیره داده‌های کاربران
user_data_store = {}  # برای اطلاعات ارسال کنندگان
last_submissions = {}  # برای محدودیت زمانی ارسال
rating_votes = {}  # جدید: برای پیگیری رأی‌دهندگان هر مدیا
user_scores = {}  # برای ذخیره امتیازهای کاربران
user_invites = {}  # برای پیگیری دعوت‌های کاربران
memory_counter = 0  # شمارنده آگهی‌های
MEMORY_COUNTER_FILE = DATA_DIR / "memory_counter.json"  # فایل ذخیره شمارنده

# ساختار جدید برای هر مدیا
class MediaRating:
    def __init__(self, ratings=None, total_score=0, vote_count=0):
        self.ratings = ratings if ratings is not None else {}  # {user_id: rating} (user_id as string)
        self.total_score = total_score
        self.vote_count = vote_count
        
    def to_dict(self):
        return {
            'ratings': {str(k): v for k, v in self.ratings.items()},  # اطمینان از ذخیره به صورت رشته
            'total_score': self.total_score,
            'vote_count': self.vote_count
        }
        
    @classmethod
    def from_dict(cls, data):
        return cls(
            ratings=data.get('ratings', {}),
            total_score=data.get('total_score', 0),
            vote_count=data.get('vote_count', 0)
        )

        
class UserScore:
    def __init__(self, total_score=0, submission_count=0, total_ratings=0, rating_count=0, invited_users=0):
        self.total_score = total_score
        self.submission_count = submission_count
        self.total_ratings = total_ratings
        self.rating_count = rating_count
        self.invited_users = invited_users
        self.last_updated = datetime.now()
        self.daily_rewards = {}  # برای پیگیری پاداش‌های روزانه

    def to_dict(self):
        return {
            'total_score': self.total_score,
            'submission_count': self.submission_count,
            'total_ratings': self.total_ratings,
            'rating_count': self.rating_count,
            'invited_users': self.invited_users,
            'last_updated': self.last_updated.isoformat(),
            'daily_rewards': self.daily_rewards  # ذخیره پاداش‌های روزانه
        }

    @classmethod
    def from_dict(cls, data):
        obj = cls(
            total_score=data.get('total_score', 0),
            submission_count=data.get('submission_count', 0),
            total_ratings=data.get('total_ratings', 0),
            rating_count=data.get('rating_count', 0),
            invited_users=data.get('invited_users', 0)
        )
        obj.daily_rewards = data.get('daily_rewards', {})  # بارگذاری پاداش‌های روزانه
        return obj
        
# تغییر حالت‌های گفتگو (در بخش StatesGroup)
class Form(StatesGroup):
    Kind = State()  # حالت جدید برای انتخاب شهر
    category = State()  # حالت جدید برای انتخاب دسته‌بندی
    contact_info = State()
    media = State()
    description = State()

class SupportForm(StatesGroup):
    issue_type = State()
    description = State()
    admin_explanation = State()  # حالت جدید برای دریافت توضیح از ادمین

def load_all_data():
    """بارگذاری تمام داده‌ها از فایل‌ها"""
    global user_data_store, user_scores, rating_votes, last_submissions, channel_posts_store, memory_counter
    
    memory_counter = load_data_from_file(MEMORY_COUNTER_FILE, 0)
    scores_data = load_data_from_file(USER_SCORES_FILE, {})
    user_scores = {}
    for user_id_str, data in scores_data.items():
        try:
            user_id = int(user_id_str)
            user_scores[user_id] = UserScore(
                total_score=data.get('total_score', 0),
                submission_count=data.get('submission_count', 0),
                total_ratings=data.get('total_ratings', 0),
                rating_count=data.get('rating_count', 0),
                invited_users=data.get('invited_users', 0)
            )
            user_scores[user_id].daily_rewards = data.get('daily_rewards', {})
        except Exception as e:
            logger.error(f"Error loading user {user_id_str}: {e}")
    
    # بارگذاری rating_votes (همانطور که قبلاً اصلاح شده بود)
    votes_data = load_data_from_file(RATING_VOTES_FILE, {})
    rating_votes = {}
    for media_id, rating_data in votes_data.items():
        try:
            if 'ratings' in rating_data:
                rating_data['ratings'] = {str(k): v for k, v in rating_data['ratings'].items()}
            rating_votes[media_id] = MediaRating.from_dict(rating_data)
        except Exception as e:
            logger.error(f"Error loading votes for {media_id}: {e}")
    
    # بارگذاری user_data_store بدون تبدیل کلیدهای غیرعددی
    user_data_store = load_data_from_file(USER_DATA_FILE, {})
    
    # تبدیل کلیدهای کاربران به عدد (فقط در سطح اول)
    if user_data_store:
        new_user_data = {}
        for k, v in user_data_store.items():
            try:
                new_key = int(k) if isinstance(k, str) and k.isdigit() else k
                new_user_data[new_key] = v
            except ValueError:
                new_user_data[k] = v
        user_data_store = new_user_data

    # تبدیل هر کاربر به لیست اگر دیکشنری است
    for user_id, data in user_data_store.items():
        if isinstance(data, dict):
            user_data_store[user_id] = [data]
    
    submissions_data = convert_keys_to_int(load_data_from_file(LAST_SUBMISSIONS_FILE, {}))
    last_submissions = {
        user_id: datetime.fromisoformat(timestamp)
        for user_id, timestamp in submissions_data.items()
    }
    
    channel_posts_store = load_data_from_file(CHANNEL_POSTS_FILE, {})

    # بارگذاری admin_messages_tracking
    admin_messages_tracking = load_data_from_file(DATA_DIR / "admin_messages_tracking.json", {})
    # بارگذاری وضعیت پذیرش قوانین (برای اطمینان از ایجاد فایل)
    _ = load_accepted_rules()

import threading
save_lock = threading.Lock()
def save_all_data():
    """ذخیره تمام داده‌ها در فایل‌ها"""
    with save_lock:
        try:
            # ذخیره امتیازات کاربران
            scores_data = {
                str(user_id): {
                    'total_score': score.total_score,
                    'submission_count': score.submission_count,
                    'total_ratings': score.total_ratings,
                    'rating_count': score.rating_count,
                    'invited_users': score.invited_users,
                    'last_updated': datetime.now().isoformat(),
                    'daily_rewards': score.daily_rewards  # ذخیره پاداش‌های روزانه
                }
                for user_id, score in user_scores.items()
            }
            save_data_to_file(scores_data, USER_SCORES_FILE)


            save_user_memories(load_user_memories())  # ذخیره آگهی‌های کاربران
            
            save_support_requests(support_requests)  # ذخیره درخواست‌های پشتیبانی

            # ذخیره شمارنده آگهی‌های
            save_data_to_file(memory_counter, MEMORY_COUNTER_FILE)

            save_user_blocks(user_blocks)  # ذخیره مسدودیت‌ها
            # بقیه ذخیره‌سازی‌ها
            save_data_to_file(user_data_store, USER_DATA_FILE)
            
            votes_data = {media_id: rating.to_dict() for media_id, rating in rating_votes.items()}
            save_data_to_file(votes_data, RATING_VOTES_FILE)
            
            submissions_data = {user_id: timestamp.isoformat() for user_id, timestamp in last_submissions.items()}
            save_data_to_file(submissions_data, LAST_SUBMISSIONS_FILE)
            
            save_data_to_file(channel_posts_store, CHANNEL_POSTS_FILE)
            
            # ذخیره admin_messages_tracking
            save_data_to_file(admin_messages_tracking, DATA_DIR / "admin_messages_tracking.json")

            logger.info("تمامی داده‌ها با موفقیت ذخیره شدند")
        except Exception as e:
            logger.error(f"Error saving all data: {e}")


from apscheduler.triggers.interval import IntervalTrigger

async def backup_data():
    try:
        save_all_data()
        logger.info("Backup completed successfully")
    except Exception as e:
        logger.error(f"Backup failed: {e}")

# در تابع on_startup:
scheduler.add_job(
    backup_data,
    IntervalTrigger(minutes=30),
    id='data_backup'
)

async def periodic_save():
    """ذخیره دوره‌ای داده‌ها"""
    try:
        save_all_data()
        logger.info("داده‌ها به صورت دوره‌ای ذخیره شدند")
    except Exception as e:
        logger.error(f"خطا در ذخیره دوره‌ای: {e}")



async def check_request_limit(user_id: int) -> bool:
    """بررسی تعداد درخواست‌های کاربر در بازه زمانی و اعمال مسدودیت در صورت تخلف"""
    current_time = datetime.now()
    
    # بررسی آیا کاربر در حال حاضر مسدود است
    if user_id in user_blocks:
        if current_time < user_blocks[user_id]:
            remaining_time = user_blocks[user_id] - current_time
            hours = int(remaining_time.total_seconds() // 3600)
            minutes = int((remaining_time.total_seconds() % 3600) // 60)
            await bot.send_message(
                user_id,
                f"⛔ شما به دلیل ارسال درخواست‌های مکرر مسدود شده‌اید.\n"
                f"🕒 مدت زمان باقی‌مانده: {hours} ساعت و {minutes} دقیقه.\n"
                f"⚠️ توجه: در صورت تکرار، مدت زمان مسدودیت افزایش می‌یابد."
            )
            return False
        else:
            del user_blocks[user_id]
            save_user_blocks(user_blocks)  # ذخیره تغییرات در فایل
    
    # بررسی تعداد درخواست‌ها در بازه زمانی
    if user_id not in user_request_timestamps:
        user_request_timestamps[user_id] = []
    
    # حذف درخواست‌های قدیمی‌تر از بازه زمانی
    user_request_timestamps[user_id] = [
        t for t in user_request_timestamps[user_id]
        if (current_time - t).total_seconds() <= REQUEST_WINDOW
    ]
    
    # اگر تعداد درخواست‌ها بیش از حد مجاز باشد
    if len(user_request_timestamps[user_id]) >= REQUEST_LIMIT:
        # افزایش تعداد تخلفات کاربر
        if user_id not in user_violations:
            user_violations[user_id] = 0
        user_violations[user_id] += 1
        
        # محاسبه مدت زمان مسدودیت (به ساعت)
        block_hours = 2 ** user_violations[user_id]  # 2, 4, 8, 16, ...
        block_end = current_time + timedelta(hours=block_hours)
        user_blocks[user_id] = block_end
        save_user_blocks(user_blocks)  # ذخیره تغییرات در فایل

        # اطلاع به کاربر
        await bot.send_message(
            user_id,
            f"⛔ شما به دلیل ارسال درخواست‌های مکرر مسدود شده‌اید.\n"
            f"🕒 مدت زمان مسدودیت: {block_hours} ساعت.\n"
            f"⚠️ توجه: در صورت تکرار، مدت زمان مسدودیت افزایش می‌یابد."
        )
        return False
    
    # ثبت درخواست جدید
    user_request_timestamps[user_id].append(current_time)
    return True

async def cleanup_old_data():
    """پاکسازی داده‌های قدیمی از حافظه"""
    current_time = datetime.now()
    
    # پاکسازی درخواست‌های قدیمی
    for user_id in list(user_request_timestamps.keys()):
        user_request_timestamps[user_id] = [
            t for t in user_request_timestamps[user_id]
            if (current_time - t).total_seconds() <= REQUEST_WINDOW
        ]
        if not user_request_timestamps[user_id]:
            del user_request_timestamps[user_id]
    
    # پاکسازی مسدودیت‌های تمام‌شده
    for user_id in list(user_blocks.keys()):
        if current_time >= user_blocks[user_id]:
            del user_blocks[user_id]
            save_user_blocks(user_blocks)  # ذخیره تغییرات در فایل


# در تابع on_startup
async def on_startup():
    """عملیات هنگام راه‌اندازی ربات"""
    global admins, blocked_users, bot_texts, user_blocks, BOT_USERNAME, report_messages, support_requests

    bot_info = await bot.get_me()
    BOT_USERNAME = bot_info.username

    # ارسال پیام زمان راه‌اندازی به ادمین اصلی
    try:
        await bot.send_message(
            ADMIN_IDM,
            f"🟢 ربات ریستارت و داده‌ها بازیابی شد.\n"
            f"⏰ زمان راه‌اندازی:\n"
            f"📅 تاریخ: {datetime.now().strftime('%Y/%m/%d')}\n"
            f"🕒 ساعت: {datetime.now().strftime('%H:%M:%S')}"
        )
    except Exception as e:
        logger.error(f"Error sending startup message: {e}")

    user_blocks = load_user_blocks()  # بارگذاری مسدودیت‌های ذخیره شده

    # بارگذاری گزارش‌ها (برای اطمینان از ایجاد فایل)
    _ = load_reports_data()

    _ = load_user_memories()  # برای اطمینان از ایجاد فایل
    
    load_all_data()
    admins = get_current_admins()
    register_admin_handlers(dp, bot, admins, blocked_users, user_blocks, DATA_DIR, ADMIN_ID, ADMIN_IDM, logger, SUPPORT_USERNAME, 
                          CHANNEL_ID, channel_posts_store, user_data_store, rating_votes, save_all_data, export_users_to_excel, generate_bot_statistics)
    
    support_requests = load_support_requests()  # بارگذاری درخواست‌های پشتیبانی

    # بارگذاری گزارش‌های فعال
    report_messages = load_report_messages()
    
    blocked_users = load_blocked_users()

    # تولید و ذخیره آمار اولیه
    try:
        stats = await generate_bot_statistics()
        stats_file = DATA_DIR / "startup_stats.json"
        with open(stats_file, 'w', encoding='utf-8') as f:
            json.dump(stats, f, ensure_ascii=False, indent=2)
        logger.info(f"آمار راه‌اندازی در {stats_file} ذخیره شد")
    except Exception as e:
        logger.error(f"Error generating startup stats: {e}")

    scheduler.add_job(
    cleanup_old_reports,
    'interval',
    hours=24,
    id='report_cleanup'
    )

    # بارگذاری اطلاعات ادمین‌ها برای ادمین‌های موجود
    admins_data = get_current_adminsdata()
    need_update = False
    
    for admin_id in admins:
        if str(admin_id) not in admins_data:
            user_info = await get_user_info(admin_id)
            if user_info:
                admins_data[str(admin_id)] = user_info
                need_update = True
    
    if need_update:
        save_admins_data(admins_data)

    scheduler.add_job(periodic_save, 'interval', minutes=5)
    
    scheduler.add_job(
        announce_winners,
        'cron',
        id='weekly_winner_announcement',  # شناسه یکتا
        day_of_week=T_day_of_week,
        hour=T_hour,
        minute=T_minute,
        second = T_second,
        timezone='Asia/Tehran',
        replace_existing=True  # جایگزینی اگر از قبل وجود داشت
    )
    

    scheduler.add_job(cleanup_old_data, 'interval', minutes=30)
    
    scheduler.start()
    logging.info(f"✅ ربات راه‌اندازی شد)")

def get_top_users(count=5):
    """محاسبه کاربران برتر با اعتبارسنجی داده‌ها"""
    valid_users = []
    
    for user_id, score in user_scores.items():
        try:
            # اعتبارسنجی داده‌های کاربر
            if not isinstance(score, UserScore):
                logger.warning(f"Invalid score type for user {user_id}")
                continue
                
            if score.total_score < 0:  # بررسی مقادیر غیرمنطقی
                logger.warning(f"Negative score for user {user_id}")
                continue
                
            valid_users.append((user_id, score.total_score))
            
        except Exception as e:
            logger.error(f"Error processing user {user_id}: {e}")
    
    # مرتب‌سازی و انتخاب برترین‌ها
    return sorted(valid_users, key=lambda x: x[1], reverse=True)[:count]

def get_week_start():
    now = datetime.now()
    return now - timedelta(days=now.weekday())  # شروع هفته جاری

# ذخیره داده‌های کاربران
user_data_store = {}
last_submissions = {}

def check_time_limit(user_id: int) -> bool:
    """بررسی فاصله زمانی بین ارسال‌ها"""
    if user_id in last_submissions:
        elapsed = (datetime.now() - last_submissions[user_id]).total_seconds()
        return elapsed < SUBMISSION_INTERVAL
    return False

# ابتدا تابع get_main_keyboard را اصلاح می‌کنیم تا دکمه امتیاز را اضافه کند
def get_main_keyboard():
    """کیبورد اصلی با دکمه‌های مورد نیاز"""
    keyboard = ReplyKeyboardMarkup(
        keyboard=[
            [KeyboardButton(text="💠 ارسال آگهی")],
            [KeyboardButton(text="🤙 ارتباط با پشتیبانی"), KeyboardButton(text="📊 آگهی‌های من")]
            # [KeyboardButton(text="📞 ارتباط با پشتیبانی"), KeyboardButton(text="📔 آگهی‌های دیگران")]
            # [KeyboardButton(text="🏆 امتیاز من"), KeyboardButton(text="👥 دعوت دوستان")]  # اضافه کردن دکمه جدید
        ],
        resize_keyboard=True
    )
    return keyboard

async def is_user_member(user_id: int) -> bool:
    """بررسی عضویت کاربر در کانال"""
    try:
        member = await bot.get_chat_member(CHANNEL_ID, user_id)
        return member.status in ['member', 'administrator', 'creator']
    except Exception as e:
        logger.error(f"Error checking membership for {user_id}: {e}")
        return False


@dp.message(lambda message: message.text == "📊 آگهی‌های من")
async def show_user_memories(message: types.Message, state: FSMContext):
    """نمایش آگهی‌های کاربر در کانال با صفحه‌بندی"""
    try:
        await state.clear()
        user_id = message.from_user.id
        
        # اعمال محدودیت درخواست
        if not await check_request_limit(user_id):
            return
            
        # بررسی عضویت در کانال
        if not await is_user_member(user_id):
            join_keyboard = InlineKeyboardMarkup(inline_keyboard=[
                [InlineKeyboardButton(text="📢 عضویت در کانال یونی مارکت", url=f"https://t.me/{CHANNEL_ID[1:]}")],
                [InlineKeyboardButton(text="✅ عضو شدم", callback_data="check_membership")]
            ])
            await message.answer(
                "🙏🏻 برای مشاهده آگهی‌ها خود، لطفاً ابتدا در کانال یونی مارکت عضو شوید:",
                reply_markup=join_keyboard
            )
            return
            
        # دریافت آگهی‌ها کاربر و مرتب‌سازی بر اساس تاریخ (جدیدترین اول)
        user_memories = load_user_memories()
        
        memories = sorted(
            user_memories.get(str(user_id), []),
            key=lambda x: x.get('timestamp', ''),
            reverse=True
        )
    
        if not memories:
            await message.answer("🍀 شما هنوز هیچ آگهی‌ای در کانال ندارید. اولین آگهی خود را ارسال کنید! 🍀")
            return
            
        # ذخیره اطلاعات صفحه‌بندی در state
        await state.update_data(
            current_page=0,
            all_memories=memories,
            total_pages=(len(memories) + 5) // 6  # 6 آگهی در هر صفحه (2 ستون × 3 ردیف)
        )
        
        # نمایش صفحه اول
        await show_memories_page(message, state)
        
    except Exception as e:
        logger.error(f"Error in show_user_memories: {e}")
        await message.answer("⚠️ خطایی در دریافت آگهی‌های شما رخ داد. لطفاً بعداً تلاش کنید.")

@dp.callback_query(lambda c: c.data in ["deleted_memory", "deleted_info"])
async def handle_deleted_memory_info(callback: types.CallbackQuery):
    """نمایش اطلاعات درباره آگهی‌های حذف شده"""
    try:
        if callback.data == "deleted_memory":
            await callback.answer("این آگهی حذف شده است", show_alert=True)
        else:
            await callback.answer(
                "آگهی‌هایی که توسط شما یا ادمین‌ها حذف شده‌اند.\n",
                show_alert=True
            )
    except Exception as e:
        logger.error(f"Error in handle_deleted_memory_info: {e}")
        await callback.answer("⚠️ خطا در نمایش اطلاعات", show_alert=True)

async def show_memories_page(message: types.Message, state: FSMContext):
    """نمایش یک صفحه از آگهی‌های کاربر"""
    data = await state.get_data()
    current_page = data.get('current_page', 0)
    all_memories = data.get('all_memories', [])
    total_pages = data.get('total_pages', 1)
    
    # محاسبه محدوده آگهی‌های برای صفحه جاری
    start_idx = current_page * 6
    end_idx = start_idx + 6
    page_memories = all_memories[start_idx:end_idx]
    
    # ایجاد دکمه‌های اینلاین (دو ستونه)
    keyboard_buttons = []
    for i in range(0, len(page_memories), 2):
        row = []
        # دکمه اول در ردیف
        if i < len(page_memories):
            memory = page_memories[i]
            btn_text = f"#{memory.get('memory_number', '?')}"
            if memory.get('status', '').startswith('deleted'):
                btn_text = f"🗑️ {btn_text}"
            row.append(
                InlineKeyboardButton(
                    text=btn_text,
                    url=memory['link'] if not memory.get('status', '').startswith('deleted') else "",
                    callback_data="deleted_memory" if memory.get('status', '').startswith('deleted') else None
                )
            )
        
        # دکمه دوم در ردیف (اگر وجود دارد)
        if i+1 < len(page_memories):
            memory = page_memories[i+1]
            btn_text = f"#{memory.get('memory_number', '?')}"
            if memory.get('status', '').startswith('deleted'):
                btn_text = f"🗑️ {btn_text}"
            row.append(
                InlineKeyboardButton(
                    text=btn_text,
                    url=memory['link'] if not memory.get('status', '').startswith('deleted') else "",
                    callback_data="deleted_memory" if memory.get('status', '').startswith('deleted') else None
                )
            )
        
        if row:
            keyboard_buttons.append(row)

    # اضافه کردن توضیح درباره نماد حذف شده
    if any(m.get('status', '').startswith('deleted') for m in page_memories):
        keyboard_buttons.append([
            InlineKeyboardButton(
                text="🗑️ = حذف شده",
                callback_data="deleted_info"
            )
        ])

    # اضافه کردن دکمه‌های صفحه‌بندی
    navigation_buttons = []
    if current_page > 0:
        navigation_buttons.append(
            InlineKeyboardButton(
                text="⬅️ صفحه قبل",
                callback_data="prev_memories_page"
            )
        )
    
    navigation_buttons.append(
        InlineKeyboardButton(
            text=f"صفحه {current_page + 1} از {total_pages}",
            callback_data="current_page_info"
        )
    )
    
    if current_page < total_pages - 1:
        navigation_buttons.append(
            InlineKeyboardButton(
                text="➡️ صفحه بعد",
                callback_data="next_memories_page"
            )
        )
    
    if navigation_buttons:
        keyboard_buttons.append(navigation_buttons)
    
    # اضافه کردن دکمه برگشت
    keyboard_buttons.append([
        InlineKeyboardButton(
            text="🔙 بازگشت به منوی اصلی",
            callback_data="return_to_main"
        )
    ])
    
    keyboard = InlineKeyboardMarkup(inline_keyboard=keyboard_buttons)
    
    # اگر پیام قبلی وجود دارد، آن را ویرایش کنید، در غیر این صورت پیام جدید ارسال کنید
    if 'page_message_id' in data:
        try:
            await bot.edit_message_text(
                chat_id=message.chat.id,
                message_id=data['page_message_id'],
                text=f"📊 آگهی‌های شما در کانال (تعداد: {len(all_memories)}):",
                reply_markup=keyboard
            )
            return
        except:
            pass
    
    # ارسال پیام جدید
    sent_message = await message.answer(
        f"📊 آگهی‌ها شما در کانال (تعداد: {len(all_memories)}):",
        reply_markup=keyboard
    )
    
    # ذخیره شناسه پیام برای ویرایش بعدی
    await state.update_data(page_message_id=sent_message.message_id)

@dp.callback_query(lambda c: c.data in ["prev_memories_page", "next_memories_page"])
async def handle_memories_pagination(callback: types.CallbackQuery, state: FSMContext):
    """پردازش تغییر صفحات آگهی‌های کاربر"""
    try:
        data = await state.get_data()
        current_page = data.get('current_page', 0)
        total_pages = data.get('total_pages', 1)
        
        if callback.data == "prev_memories_page" and current_page > 0:
            current_page -= 1
        elif callback.data == "next_memories_page" and current_page < total_pages - 1:
            current_page += 1
        
        await state.update_data(current_page=current_page)
        await show_memories_page(callback.message, state)
        await callback.answer()
        
    except Exception as e:
        logger.error(f"Error in handle_memories_pagination: {e}")
        await callback.answer("⚠️ خطایی در تغییر صفحه رخ داد.")

@dp.callback_query(lambda c: c.data == "current_page_info")
async def handle_page_info(callback: types.CallbackQuery):
    """پردازش نمایش اطلاعات صفحه جاری"""
    await callback.answer("شما در این صفحه هستید")

@dp.callback_query(lambda c: c.data == "return_to_main")
async def return_to_main(callback: types.CallbackQuery, state: FSMContext):
    """بازگشت به منوی اصلی"""
    try:
        user_id = callback.from_user.id
        admins = get_current_admins()
        
        # پاکسازی state
        await state.clear()
        
        # حذف پیام صفحه‌بندی
        try:
            await callback.message.delete()
        except:
            pass
        
        if user_id in admins:
            await callback.message.answer(
                "✅ به منوی اصلی بازگشتید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await callback.message.answer(
                "✅ به منوی اصلی بازگشتید.",
                reply_markup=get_main_keyboard()
            )
            
    except Exception as e:
        logger.error(f"Error in return_to_main: {e}")
        await callback.answer("⚠️ خطایی رخ داد. لطفاً از منوی اصلی شروع کنید.")


from openpyxl.styles import Font
import openpyxl
async def export_users_to_excel():
    """صادر کردن اطلاعات کاربران به فایل اکسل"""
    try:
        
        # ایجاد یک workbook جدید
        wb = openpyxl.Workbook()
        ws = wb.active
        ws.title = "Users Data"
        
        # ایجاد هدرهای ستون‌ها
        headers = [
            "نام کامل", 
            "آیدی عددی", 
            "یوزرنیم", 
            "وضعیت بلاک", 
            "وضعیت ادمین", 
            "تعداد آگهی‌ها", 
            "تعداد دعوت‌ها", 
            "امتیاز کل", 
            "آخرین فعالیت (تاریخ)",
            "آخرین فعالیت (زمان)"

        ]
        
        ws.append(headers)
        admins = get_current_admins()
        # استایل برای هدرها
        for cell in ws[1]:
            cell.font = Font(bold=True)
        
        # جمع‌آوری تمام کاربران از منابع مختلف
        all_user_ids = set()
        all_user_ids.update(user_data_store.keys())
        all_user_ids.update(user_scores.keys())
        # all_user_ids.update(rating_votes.keys())
        
        # اطمینان حاصل کنید که فقط اعداد صحیح هستند (آیدی کاربران)
        all_user_ids = {uid for uid in all_user_ids if isinstance(uid, int)}

        # پر کردن داده‌ها
        for user_id in all_user_ids:
            try:
                # دریافت اطلاعات کاربر
                user_info = await get_user_info(user_id)
                
                # اطلاعات پایه
                full_name = ""
                username = ""
                
                if user_info:
                    full_name = f"{user_info.get('first_name', '')} {user_info.get('last_name', '')}".strip()
                    username = user_info.get('username', '')
                
                # اطلاعات وضعیت
                is_blocked = "بله" if user_id in blocked_users else "خیر"
                is_admin = "بله" if user_id in admins else "خیر"
                
                # اطلاعات امتیازات
                user_score = user_scores.get(user_id, UserScore())
                if isinstance(user_score, dict):
                    user_score = UserScore.from_dict(user_score)
                
                submission_count = user_score.submission_count
                invited_users = user_score.invited_users
                total_score = calculate_user_score(user_id)
                
                # آخرین فعالیت
                last_activityD = ""
                last_activityT = ""
                if user_id in last_submissions:
                    last_activityD = last_submissions[user_id].strftime("%Y-%m-%d")
                    last_activityT = last_submissions[user_id].strftime("%H:%M")
                
                # افزودن ردیف به اکسل
                ws.append([
                    full_name,
                    user_id,
                    f"@{username}" if username else "ندارد",
                    is_blocked,
                    is_admin,
                    submission_count,
                    invited_users,
                    total_score,
                    last_activityD,
                    last_activityT
                ])
                
            except Exception as e:
                logger.error(f"Error processing user {user_id}: {e}")
                continue
        
        # تنظیم عرض ستون‌ها
        column_widths = {
            'A': 25,  # نام کامل
            'B': 15,  # آیدی عددی
            'C': 20,  # یوزرنیم
            'D': 12,  # وضعیت بلاک
            'E': 12,  # وضعیت ادمین
            'F': 15,  # تعداد آگهی‌ها
            'G': 15,  # تعداد دعوت‌ها
            'H': 12,  # امتیاز کل
            'I': 15,   # آخرین فعالیت (تاریخ)
            'J': 10    # آخرین فعالیت (زمان)
        }
        
        for col, width in column_widths.items():
            ws.column_dimensions[col].width = width
        
        # ذخیره فایل
        timestamp = datetime.now().strftime("%Y%m%d_%H%M")
        filename = DATA_DIR / f"users_data_{timestamp}.xlsx"
        
        wb.save(filename)
        logger.info(f"Excel file saved: {filename}")
        
        return filename
        
    except Exception as e:
        logger.error(f"Error in export_users_to_excel: {e}")
        return None
    

@dp.callback_query(lambda c: c.data == "check_membership")
async def check_membership(callback: types.CallbackQuery):
    """بررسی مجدد عضویت کاربر در کانال"""
    try:
        user_id = callback.from_user.id
        is_member = await is_user_member(user_id)
        
        if is_member:
            await callback.message.delete()
            welcome_message = bot_texts["welcome_text"]

            if user_id in admins:
                await callback.message.answer(welcome_message, reply_markup=get_admin_keyboard())
            else:
                await callback.message.answer(welcome_message, reply_markup=get_main_keyboard())

        else:
            join_keyboard = InlineKeyboardMarkup(inline_keyboard=[
                [InlineKeyboardButton(text="📢 عضویت در کانال یونی مارکت", url=f"https://t.me/{CHANNEL_ID[1:]}")],
                [InlineKeyboardButton(text="✅ عضو شدم", callback_data="check_membership")]
            ])
            await callback.answer("😅 شما هنوز در کانال عضو نشدید!", show_alert=True)
            await callback.message.edit_reply_markup(reply_markup=join_keyboard)
            
    except Exception as e:
        logger.error(f"Error in check_membership: {e}")
        await callback.answer("⚠️ خطایی رخ داد. لطفاً مجدداً تلاش کنید.")


@dp.message(Command("start"))
async def start(message: types.Message, state: FSMContext):
    """شروع ربات با پیام خوشامد و دکمه‌ها"""
    try:
        user_id = message.from_user.id
        admins = get_current_admins()
        # بررسی وضعیت بلاک کاربر
        if is_user_blocked(user_id):
            await message.answer("⛔ متأسفانه شما توسط ادمین بلاک شده‌اید و نمی‌توانید از ربات استفاده کنید. \n 🍀در صورت نیاز به توضیح با پشتیبانی ارتباط بگیرید.🍀")
            return
        
        # اعمال محدودیت درخواست
        if not await check_request_limit(message.from_user.id):
            return

        # ذخیره اطلاعات اولیه کاربر
        is_new_user = user_id not in user_data_store
        if is_new_user:
            user_data_store[user_id] = {
                'user_id': user_id,
                'name': message.from_user.full_name,
                'username': message.from_user.username or '',
                'submission_time': datetime.now(),
                'first_join_date': datetime.now().isoformat(),
                'invited_by': None
            }

        # بررسی عضویت کاربر در کانال
        is_member = await is_user_member(user_id)
        
        # پردازش لینک دعوت
        if len(message.text.split()) > 1 and message.text.split()[1].startswith("invite_"):
            try:
                inviter_id = int(message.text.split()[1].split('_')[1])
                
                # جلوگیری از دعوت خودکار
                if inviter_id == user_id:
                    await message.answer("❌ نمی‌توانید خودتان را دعوت کنید!")
                else:
                    # فقط اگر کاربر جدید باشد و قبلاً توسط کسی دعوت نشده باشد
                    if is_new_user and user_data_store[user_id]['invited_by'] is None:
                        user_data_store[user_id]['invited_by'] = inviter_id
                        
                        # افزایش امتیاز دعوت‌کننده
                        if inviter_id not in user_scores:
                            user_scores[inviter_id] = UserScore()
                        
                        user_scores[inviter_id].invited_users += 1
                        user_scores[inviter_id].total_score += INVITE_REWARD
                        
                        # اطلاع به دعوت‌کننده
                        try:
                            invited_user_name = message.from_user.first_name
                            invited_user_username = f"@{message.from_user.username}" if message.from_user.username else "ندارد"

                            await bot.send_message(
                                inviter_id,
                                f"🎉 کاربر جدید با مشخصات زیر با لینک دعوت شما به ربات پیوست:\n\n"
                                f"👤 نام: {invited_user_name}\n"
                                f"📌 یوزرنیم: {invited_user_username}\n"
                                f"🆔 آیدی: {user_id}\n\n"
                                f"شما {INVITE_REWARD} امتیاز دریافت کردید!"
                            )

                        except Exception as e:
                            logger.error(f"خطا در اطلاع به دعوت‌کننده: {e}")
            except Exception as e:
                logger.error(f"خطا در پردازش لینک دعوت: {e}")
                # اگر خطایی در پردازش لینک دعوت رخ داد، ادامه دهد بدون قطع شدن

        welcome_message = bot_texts["welcome_text"]
        
        if not is_member:
            join_keyboard = InlineKeyboardMarkup(inline_keyboard=[
                [InlineKeyboardButton(text="📢 عضویت در کانال یونی مارکت", url=f"https://t.me/{CHANNEL_ID[1:]}")],
                [InlineKeyboardButton(text="✅ عضو شدم", callback_data="check_membership")]
            ])
            await message.answer(
                "🙏🏻 برای استفاده از ربات، لطفاً ابتدا در کانال یونی مارکت عضو شوید:",
                reply_markup=join_keyboard
            )
        else:
            # اگر کاربر ادمین است، کیبورد ادمین را نشان بده
            if user_id in admins:
                await message.answer(welcome_message, reply_markup=await get_admin_keyboard())
            else:
                await message.answer(welcome_message, reply_markup=get_main_keyboard())
        
        save_all_data()

    except Exception as e:
        logger.error(f"خطای شدید در تابع start: {e}", exc_info=True)
        await message.answer("⚠️ خطایی در ورود به ربات رخ داد. لطفاً بعداً مجدداً تلاش کنید.")

def get_cancel_keyboard():
    """کیبورد لغو و بازگشت به منوی اصلی"""
    keyboard = ReplyKeyboardMarkup(
        keyboard=[
            [KeyboardButton(text="🔙 بازگشت به منوی اصلی")]
        ],
        resize_keyboard=True
    )
    return keyboard

@dp.message(lambda message: message.text == "🔙 بازگشت به منوی اصلی")
async def cancel_process(message: types.Message, state: FSMContext):
    try:
        user_id = message.from_user.id
        admins = get_current_admins()
        # پاک کردن کامل state و داده‌های موقت
        current_state = await state.get_state()
        if current_state:
            await state.clear()

        # حذف آخرین ارسال از لیست last_submissions
        if user_id in last_submissions:
            del last_submissions[user_id]

        # اطمینان از بازگشت به حالت اولیه
        if message.from_user.id in admins:
            await message.answer(
                "✅ به منوی اصلی بازگشتید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await message.answer(
                "✅ به منوی اصلی بازگشتید.",
                reply_markup=get_main_keyboard()
            )
        
    except Exception as e:
        logger.error(f"Error in cancel_process: {e}", exc_info=True)


        if user_id in admins:
            await message.answer(
                "⚠️ خطایی رخ داد. لطفاً مجدداً از منوی اصلی شروع کنید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await message.answer(
                "⚠️ خطایی رخ داد. لطفاً مجدداً از منوی اصلی شروع کنید.",
                reply_markup=get_main_keyboard()
            )


def seconds_to_hms(seconds: int) -> str:
    """تبدیل ثانیه به فرمت ساعت:دقیقه:ثانیه"""
    hours = seconds // 3600
    minutes = (seconds % 3600) // 60
    secs = seconds % 60
    return f"{hours:02d}:{minutes:02d}:{secs:02d}"


@dp.message(lambda message: message.text == "💠 ارسال آگهی")
async def start_submission(message: types.Message, state: FSMContext):
    """شروع فرآیند ارسال رسانه"""
    try:
        
        user_id = message.from_user.id
        # پاکسازی کامل state قبل از شروع
        await state.clear()
        admins = get_current_admins()
        # بررسی وضعیت بلاک کاربر
        if is_user_blocked(user_id):
            await message.answer("⛔ متأسفانه شما توسط ادمین بلاک شده‌اید و نمی‌توانید از ربات استفاده کنید. \n 🍀در صورت نیاز به توضیح با پشتیبانی ارتباط بگیرید.🍀")
            return
        
        # بررسی محدودیت روزانه
        if check_daily_submission_limit(user_id):
            if user_id in admins:
                await message.answer(
                    f"🙏🏻 هر کاربر در هر روز {DAILY_SUBMISSION_LIMIT} آگهی می‌تواند ارسال کند.\n"
                    "🍀شما به سقف مجاز ارسال آگهی رسیدید. لطفاً فردا مجدداً تلاش کنید.🍀",
                    reply_markup=await get_admin_keyboard()
                )
            else:
                await message.answer(
                    f"🙏🏻 هر کاربر در هر روز {DAILY_SUBMISSION_LIMIT} آگهی می‌تواند ارسال کند.\n"
                    "🍀شما به سقف مجاز ارسال آگهی رسیدید. لطفاً فردا مجدداً تلاش کنید.🍀",
                    reply_markup=get_main_keyboard()
                )


            return
        

        # اعمال محدودیت درخواست
        if not await check_request_limit(message.from_user.id):
            return
        # بررسی عضویت کاربر در کانال
        is_member = await is_user_member(user_id)
        if not is_member:
            join_keyboard = InlineKeyboardMarkup(inline_keyboard=[
                [InlineKeyboardButton(text="📢 عضویت در کانال یونی مارکت", url=f"https://t.me/{CHANNEL_ID[1:]}")],
                [InlineKeyboardButton(text="✅ عضو شدم", callback_data="check_membership")]
            ])
            await message.answer(
                "🙏🏻 برای ارسال آگهی، لطفاً ابتدا در کانال یونی مارکت عضو شوید:",
                reply_markup=join_keyboard
            )
            return
        
        if check_time_limit(user_id):
            remaining = SUBMISSION_INTERVAL - (datetime.now() - last_submissions[user_id]).total_seconds()
            remaining_int = int(remaining)
            remaining_time = seconds_to_hms(remaining_int)
            await message.answer(f"⏳ شما اخیراً آگهی‌ای ارسال کرده‌اید. لطفاً آگهی بعدی خود را {remaining_time} دیگر ارسال کنید.")
            return
        
        # بررسی پذیرش قوانین
        if not has_accepted_rules(user_id):
            rules_keyboard = InlineKeyboardMarkup(inline_keyboard=[
                [InlineKeyboardButton(text="✅ تمام قوانین بالا را می‌پذیرم", callback_data="accept_rules")]
            ])
            await message.answer(
                bot_texts["rules_text"],
                reply_markup=rules_keyboard
            )
            return
        
        # شروع فرآیند جدید با انتخاب شهر
        keyboard = ReplyKeyboardMarkup(
            keyboard=[
                [KeyboardButton(text="🛒 فروشنده (ارائه)"), KeyboardButton(text="💰 خریدار (درخواست)")],
                [KeyboardButton(text="🔙 بازگشت به منوی اصلی")]
            ],
            resize_keyboard=True
        )
        await message.answer("لطفاً انتخاب کنید که فروشنده هستید یا خریدار:", reply_markup=keyboard)
        await state.set_state(Form.Kind)
        
    except Exception as e:
        logger.error(f"Error in start_submission: {e}", exc_info=True)
        await state.clear()
        if user_id in admins:
            await message.answer(
                "⚠️ خطا در شروع فرآیند ارسال. لطفاً مجدداً تلاش کنید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await message.answer(
                "⚠️ خطا در شروع فرآیند ارسال. لطفاً مجدداً تلاش کنید.",
                reply_markup=get_main_keyboard()
            )

@dp.message(Form.Kind)
async def process_Kind(message: types.Message, state: FSMContext):
    try:
        if message.text == "🔙 بازگشت به منوی اصلی":
            await cancel_process(message, state)
            return
            
        Kind = message.text
        if Kind not in ["💰 خریدار (درخواست)", "🛒 فروشنده (ارائه)"]:
            await message.answer("لطفاً فقط یکی از گزینه‌های 'فروشنده' یا 'خریدار' را انتخاب کنید.")
            return
            
        await state.update_data(Kind=Kind)
        
        # نمایش دسته‌بندی‌های مربوط به شهر انتخاب شده به صورت دو ستونه
        categories = CATEGORIES
        keyboard_buttons = []
        
        # تقسیم دسته‌بندی‌ها به دو ستون
        for i in range(0, len(categories), 2):
            row = []
            if i < len(categories):
                row.append(KeyboardButton(text=categories[i]))
            if i+1 < len(categories):
                row.append(KeyboardButton(text=categories[i+1]))
            if row:
                keyboard_buttons.append(row)
        
        # اضافه کردن دکمه بازگشت
        keyboard_buttons.append([KeyboardButton(text="🔙 بازگشت به منوی اصلی")])
        
        keyboard = ReplyKeyboardMarkup(
            keyboard=keyboard_buttons,
            resize_keyboard=True
        )
        
        await message.answer("لطفاً دسته‌بندی مربوط به آگهی خود را انتخاب کنید:", reply_markup=keyboard)
        await state.set_state(Form.category)
        
    except Exception as e:
        logger.error(f"Error in process_Kind: {e}")
        await state.clear()
        await message.answer(
            "⚠️ خطایی رخ داد. لطفاً مجدداً از منوی اصلی شروع کنید.",
            reply_markup=get_main_keyboard()
        )

@dp.message(Form.category)
async def process_category(message: types.Message, state: FSMContext):
    try:
        if message.text == "🔙 بازگشت به منوی اصلی":
            await cancel_process(message, state)
            return
            
        data = await state.get_data()
        
        valid_categories = CATEGORIES
            
        if message.text not in valid_categories:
            await message.answer(f"لطفاً یکی از دسته‌بندی‌های معتبر را انتخاب کنید: {', '.join(valid_categories)}")
            return
            
        await state.update_data(category=message.text)
        
        # ادامه فرآیند مانند قبل
        if message.from_user.username:
            keyboard = InlineKeyboardMarkup(inline_keyboard=[
                [InlineKeyboardButton(text="✅ بله", callback_data="publish_username_yes"),
                 InlineKeyboardButton(text="❌ خیر", callback_data="publish_username_no")],
                [InlineKeyboardButton(text="🔙 بازگشت به منوی اصلی", callback_data="cancel_submission")]
            ])
            
            await message.answer(
                f"👤 آیا مایلید یوزرنیم تلگرام شما (@{message.from_user.username}) همراه آگهی منتشر شود؟",
                reply_markup=keyboard
            )
        else:
            await state.set_state(Form.contact_info)
            await message.answer(
                "📞 لطفاً یک راه ارتباطی برای نمایش در آگهی ارائه دهید:\n\n"
                "• میتوانید یک یوزرنیم دیگر (مثال: @Example)\n"
                "• یا شماره تلفن همراه (مثال: 09123456789)\n"
                "• یا هر روش ارتباطی دیگری را وارد کنید\n\n"
                "این اطلاعات به صورت عمومی در کانال نمایش داده خواهد شد.",
                reply_markup=get_cancel_keyboard()
            )
            
    except Exception as e:
        logger.error(f"Error in process_category: {e}")
        await state.clear()
        await message.answer(
            "⚠️ خطایی رخ داد. لطفاً مجدداً از منوی اصلی شروع کنید.",
            reply_markup=get_main_keyboard()
        )


@dp.callback_query(lambda c: c.data == "cancel_submission")
async def cancel_submission(callback: types.CallbackQuery, state: FSMContext):
    await cancel_process(callback.message, state)
    await callback.message.delete()


@dp.message(Form.contact_info)
async def get_contact_info(message: types.Message, state: FSMContext):
    """دریافت اطلاعات تماس از کاربر"""
    try:
        if message.text == "🔙 بازگشت به منوی اصلی":
            await cancel_process(message, state)
            return
            
        if message.text == "⏭️ این آگهی عکس یا ویدیو ندارد":
            await state.update_data(contact_info="ارسال نشده")
            await state.set_state(Form.description)
            await message.answer(
                "📝 لطفاً توضیحی درباره این آگهی بنویسید:",
                reply_markup=get_cancel_keyboard()
            )
            return

        contact_info = message.text.strip()
        
        # اعتبارسنجی اطلاعات تماس
        if len(contact_info) < 3 or len(contact_info) > 50:
            await message.answer(
                "❌ اطلاعات تماس باید بین 3 تا 50 کاراکتر باشد. لطفاً مجدداً وارد کنید:",
                reply_markup=get_cancel_keyboard()
            )
            return
            
        # بررسی کلمات ممنوعه
        words_in_contact = contact_info.lower().split()
        if any(word in banned_words for word in words_in_contact):
            await message.answer(
                "❌ اطلاعات تماس حاوی کلمات ممنوعه است.\n لطفاً آن را اصلاح کنید.",
                reply_markup=get_cancel_keyboard()
            )
            return

        await state.update_data(contact_info=contact_info)
        
        # ایجاد کیبورد با گزینه رد شدن از رسانه
        keyboard = ReplyKeyboardMarkup(
            keyboard=[
                [KeyboardButton(text="⏭️ این آگهی عکس یا ویدیو ندارد")],
                [KeyboardButton(text="🔙 بازگشت به منوی اصلی")]
            ],
            resize_keyboard=True
        )
        
        await message.answer(
            "📸 لطفاً یک عکس یا ویدیوی مربوط به آگهی خود را ارسال کنید:\n"
            "یا اگر مایلید بدون رسانه ادامه دهید، روی «⏭️ این آگهی عکس یا ویدیو ندارد» کلیک کنید.\n\n"
            "🍀 نکات مهم:\n"
            "- حجم فایل باید کمتر از 50 مگابایت باشد\n"
            "- فقط یک رسانه می‌توانید ارسال کنید\n"
            "- از نوشتن کپشن در این مرحله خودداری کنید",
            reply_markup=keyboard
        )
        await state.set_state(Form.media)
        
    except Exception as e:
        logger.error(f"Error in get_contact_info: {e}")
        await state.clear()
        await message.answer(
            "⚠️ خطایی رخ داد. لطفاً مجدداً از منوی اصلی شروع کنید.",
            reply_markup=get_main_keyboard()
        )
        
@dp.message(lambda message: message.text == "🤙 ارتباط با پشتیبانی")
async def contact_support(message: types.Message, state: FSMContext):
    """ارتباط با پشتیبانی"""
    try:
        user_id = message.from_user.id
        
        # اعمال محدودیت درخواست
        if not await check_request_limit(user_id):
            return
        
        # بررسی محدودیت درخواست‌های پشتیبانی
        if not check_support_request_limit(user_id):
            remaining_time = "24 ساعت"  # تا فردا همین وقت

            if user_id in admins:
                await message.answer(
                    f"⏳ شما امروز 3 درخواست پشتیبانی ارسال کرده‌اید.\n"
                    f"🍀لطفاً پس از {remaining_time} مجدداً تلاش کنید.🍀",
                    reply_markup=await get_admin_keyboard()
                )
                return
            else:
                await message.answer(
                    f"⏳ شما امروز 3 درخواست پشتیبانی ارسال کرده‌اید.\n"
                    f"🍀لطفاً پس از {remaining_time} مجدداً تلاش کنید.🍀",
                    reply_markup=get_main_keyboard()
                )
                return

        keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [InlineKeyboardButton(text="🤖 ربات من را اشتباهی موقتا مسدود کرده است.", callback_data="support_blocked")],
            [InlineKeyboardButton(text="👤 ادمین من را اشتباهی بلاک کرده است.", callback_data="support_admin_blocked")],
            [InlineKeyboardButton(text="⚠️ آخرین آگهی من اشتباهی اخطار گرفته است.", callback_data="support_warned")],
            [InlineKeyboardButton(text="❌ آخرین آگهی من اشتباهی رد شده است.", callback_data="support_rejected")],
            [InlineKeyboardButton(text="👮‍♂️ از من کلاهبرداری شده است.", callback_data="support_police")],
            [InlineKeyboardButton(text="👑 میخواهم ادمین 'یونی مارکت' باشم.", callback_data="support_admin_request")],
            [InlineKeyboardButton(text="💡 پیشنهاد یا انتقادی برای ربات و کانال دارم.", callback_data="support_feedback")],
            [InlineKeyboardButton(text="🔍 آگهی من از کانال پس از تایید حذف شده است.", callback_data="support_post_deleted")],
            [InlineKeyboardButton(text="📊 در منوی '📊 آگهی‌های من' آگهی خود را نمی‌بینم.", callback_data="support_Ads")],
            [InlineKeyboardButton(text="❓ سایر مسائل", callback_data="support_other")]
        ])
        
        await message.answer(bot_texts["support_text"], reply_markup=keyboard)
        await message.delete()  # حذف پیام اصلی
        await state.set_state(SupportForm.issue_type)
    except Exception as e:
        logger.error(f"Error in contact_support: {e}")
        await message.answer("⚠️ خطایی در ارتباط با پشتیبانی رخ داد.")

@dp.callback_query(lambda c: c.data.startswith('support_') and not c.data.startswith(('support_done_', 'support_rejected_')))
async def handle_support_issue(callback: types.CallbackQuery, state: FSMContext):
    """پردازش انتخاب نوع مشکل از سوی کاربر"""
    try:
        issue_type = callback.data
        await state.update_data(issue_type=issue_type)
        
        user_id = callback.from_user.id

        # حذف پیام دکمه‌های شیشه‌ای
        await callback.message.delete()

        
        if user_id in admins:
            await callback.message.answer("لطفاً مشکل را به صورت متن و کامل شرح دهید: \n 🍀لطفاً به هیچ عنوان تصویر یا فیلم ارسال نکنید.🍀",
                reply_markup=await get_admin_keyboard())
        else:
            await callback.message.answer("لطفاً مشکل را به صورت متن و کامل شرح دهید: \n 🍀لطفاً به هیچ عنوان تصویر یا فیلم ارسال نکنید.🍀",
                reply_markup=get_main_keyboard())
        

        await state.set_state(SupportForm.description)
    except Exception as e:
        logger.error(f"Error in handle_support_issue: {e}")
        await callback.message.answer("⚠️ خطایی در ثبت مشکل رخ داد.")

@dp.callback_query(lambda c: c.data.startswith(('support_done_', 'support_rejected_')))
async def handle_support_response(callback: types.CallbackQuery, state: FSMContext):
    """پردازش پاسخ ادمین به درخواست پشتیبانی"""
    try:
        # بررسی ادمین بودن کاربر
        if callback.from_user.id not in get_current_admins():
            await callback.answer("⛔ شما دسترسی به این عملیات را ندارید.", show_alert=True)
            return
            
        # پردازش callback_data
        parts = callback.data.split('_')
        if len(parts) != 3:
            logger.error(f"Invalid callback_data format: {callback.data}")
            await callback.answer("⚠️ خطایی در پردازش پاسخ رخ داد.", show_alert=True)
            return
            
        action = parts[1]  # done یا rejected
        user_id = int(parts[2])
        
        # ذخیره اطلاعات در حالت
        await state.update_data(
            action=action,
            user_id=user_id,
            support_data=support_messages[user_id] if user_id in support_messages else None
        )
        
        # درخواست توضیح از ادمین
        await callback.message.answer("لطفاً اگر پیامی برای کاربر دارید، توضیحات خود را بنویسید. \n 🍀لطفاً به هیچ عنوان تصویر یا فیلم ارسال نکنید.🍀")
        await state.set_state(SupportForm.admin_explanation)
        
    except Exception as e:
        logger.error(f"Error in handle_support_response: {e}")
        await callback.answer("⚠️ خطایی در ثبت پاسخ رخ داد.", show_alert=True)

@dp.message(SupportForm.admin_explanation)
async def handle_admin_explanation(message: types.Message, state: FSMContext):
    """پردازش توضیح ادمین"""
    try:
        data = await state.get_data()
        action = data.get('action')
        user_id = data.get('user_id')
        support_data = data.get('support_data')
        
        if not all([action, user_id, support_data]):
            await message.answer("⚠️ خطایی در پردازش اطلاعات رخ داد.")
            await state.clear()
            return
            
        admin_explanation = message.text
        status_text = "✅ انجام شد" if action == "done" else "❌ رد شد"
        admin_name = message.from_user.full_name
        if message.from_user.username:
            admin_name = f"@{message.from_user.username}"
            
        # ویرایش پیام‌های ادمین‌ها
        for admin_message in support_data['messages']:
            try:
                original_text = admin_message.text
                new_text = f"{original_text}\n\n🔄 وضعیت: {status_text}\n👤 ادمین: {admin_name}\n📝 توضیح ادمین:\n{admin_explanation}"
                await admin_message.edit_text(new_text, reply_markup=None)
            except Exception as e:
                logger.error(f"Error updating admin message: {e}")
                
        # ارسال پیام به کاربر
        try:
            await message.bot.send_message(
                chat_id=user_id,
                text=f"📝 وضعیت درخواست پشتیبانی شما:\n\n"
                     f"عنوان مشکل: {support_data['issue_type']}\n\n"
                     f"توضیح شما: {support_data['description']}\n\n"
                     f"🔄 وضعیت: {status_text}\n"
                     f"📝 توضیح ادمین:\n{admin_explanation}"
            )
            logger.info(f"Response sent to user {user_id} by admin {message.from_user.id}")
        except Exception as e:
            logger.error(f"Error sending response to user {user_id}: {e}")
            await message.answer("⚠️ خطایی در ارسال پاسخ به کاربر رخ داد.")
            return
            
        # پاک کردن پیام‌ها از دیکشنری
        if user_id in support_messages:
            del support_messages[user_id]
            
        await message.answer("✅ پاسخ شما به درخواست کاربر با موفقیت ثبت و ارسال شد.")
        await state.clear()
        
    except Exception as e:
        logger.error(f"Error in handle_admin_explanation: {e}")
        await message.answer("⚠️ خطایی در ثبت پاسخ رخ داد.")
        await state.clear()

@dp.message(lambda message: message.text == "📔 آگهی‌های دیگران")
async def join_channel(message: types.Message):
    """ورود مستقیم به کانال"""
    try:
        # اعمال محدودیت درخواست
        if not await check_request_limit(message.from_user.id):
            return
        # ساخت دکمه اینلاین با لینک مستقیم به کانال
        keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [InlineKeyboardButton(text="📔 مشاهده آگهی‌ها", url=f"https://t.me/{CHANNEL_ID[1:]}")]
        ])
        
        await message.answer(
            "⬇️➖➖➖➖➖➖➖➖⬇️",
            reply_markup=keyboard
        )
    except Exception as e:
        logger.error(f"Error in join_channel: {e}")
        await message.answer("⚠️ خطایی در نمایش لینک کانال رخ داد.")


# دیکشنری برای ذخیره پیام‌های پشتیبانی
support_messages = {}
# در بخش تنظیمات ربات
USER_RANKS = [
    {"min_score": 0, "name": "میهمان حافظیه", "icon": "🌸"},
    {"min_score": 50, "name": "دوستدار شیراز", "icon": "🌹"},
    {"min_score": 200, "name": "نسیم بهارنارنج", "icon": "🌿"},
    {"min_score": 500, "name": "شیرازشناس", "icon": "🏛️"},
    {"min_score": 1000, "name": "قصه‌گوی سعدی", "icon": "📜"},
    {"min_score": 2000, "name": "خیال حافظ", "icon": "🕯️"},
    {"min_score": 5000, "name": "خورشید خاطرات", "icon": "☀️"},
    {"min_score": 20000, "name": "گل‌گشت ارم", "icon": "🌹"},
    {"min_score": 50000, "name": "سایه‌ی سرو شیراز", "icon": "🌳"},
    {"min_score": 100000, "name": "شیرازی اصیل", "icon": "🦁"}
]

def get_user_rank(user_id: int) -> dict:
    """تعیین رتبه کاربر بر اساس امتیاز"""
    score = calculate_user_score(user_id)
    for rank in reversed(USER_RANKS):
        if score >= rank["min_score"]:
            return rank
    return USER_RANKS[0]  # حالت پیش‌فرض


# سپس تابع show_score را تغییر می‌دهیم تا با دکمه کار کند (به جای کامند)
@dp.message(lambda message: message.text == "🏆 امتیاز من")
async def show_score(message: types.Message):
    try:
        user_id = message.from_user.id
        if user_id not in user_scores:
            user_scores[user_id] = UserScore()
            
        current_score = calculate_user_score(user_id)
        rank = get_user_rank(user_id)
        next_rank = None
        
        # پیدا کردن رتبه بعدی
        for r in USER_RANKS:
            if r["min_score"] > current_score:
                next_rank = r
                break
        
        progress = ""
        if next_rank:
            progress_percent = min(100, int((current_score / next_rank["min_score"]) * 100))
            # ساخت نوار پیشرفت گرافیکی
            filled = '█' * int(progress_percent / 10)
            empty = '░' * (10 - len(filled))
            progress_bar = f"{filled}{empty}"
            
            progress = (
                f"\n\n📊 پیشرفت شما در رتبه فعلی:\n"
                f"{progress_bar} {progress_percent}%\n"
                f"🎉 برای رتبه بعدی ({next_rank['icon']} {next_rank['name']}) شما به {next_rank['min_score'] - current_score} امتیاز دیگر نیاز دارید.\n"
            )
        
        await message.answer(
            f"{rank['icon']} رتبه فعلی شما: {rank['name']}\n"
            f"💎 امتیاز کل: {current_score:.1f}\n\n"
            f"📤 تعداد آگهی‌های شما: {user_scores[user_id].submission_count}\n"
            f"⭐ مجموع امتیاز آگهی‌ها: {user_scores[user_id].total_ratings}\n"
            f"👥 تعداد دعوت‌ها: {user_scores[user_id].invited_users}\n"
            f"⭐ مجموع امتیاز دعوت‌ها: {user_scores[user_id].invited_users * INVITE_REWARD}\n"
            f"🎁 امتیاز پاداش روزانه: {user_scores[user_id].total_score}\n"
            f"{progress}",
            parse_mode="HTML"
        )

    except Exception as e:
        logger.error(f"Error in show_score: {e}")
        await message.answer("⚠️ خطا در محاسبه امتیاز!")

# سپس تابع invite_friends را تغییر می‌دهیم تا با دکمه کار کند (به جای کامند)
@dp.message(lambda message: message.text == "👥 دعوت دوستان")
async def invite_friends(message: types.Message):
    try:
        user_id = message.from_user.id
        
        if is_user_blocked(user_id):
            await message.answer("⛔ متأسفانه شما توسط ادمین بلاک شده‌اید و نمی‌توانید از ربات استفاده کنید. \n 🍀در صورت نیاز به توضیح با پشتیبانی ارتباط بگیرید.🍀")
            return
        # اعمال محدودیت درخواست
        if not await check_request_limit(message.from_user.id):
            return
        bot_info = await bot.get_me()
        bot_username = bot_info.username

        invite_link = f"https://t.me/{bot_username}?start=invite_{user_id}"

        await message.answer(
            f"👥 دوستان خود را به ربات دعوت کنید و برای هر دعوت موفق {INVITE_REWARD} امتیاز دریافت کنید!\n\n"
            f"🔗 لینک دعوت اختصاصی شما:\n{invite_link}\n\n"
            "📌 شرایط دریافت امتیاز:\n"
            "1. فقط دعوت کاربران جدید امتیاز دارد\n"
            "2. کاربرانی که قبلاً ربات را استارت کرده‌اند، حتی اگر از لینک شما استفاده کنند، امتیازی برای شما ندارند\n"
            "3. هر کاربر فقط برای یک دعوت‌کننده امتیاز محاسبه می‌شود\n"
            "4. دعوت خودتان امتیازی ندارد\n\n"
        )
    except Exception as e:
        logger.error(f"Error in invite_friends: {e}")


import re

# @dp.message(Form.name)
# async def get_name(message: types.Message, state: FSMContext):
#     """دریافت نام کاربر"""
#     try:
#         name = message.text.strip()
#         user_id = message.from_user.id
#         admins = get_current_admins()

#         # اعتبارسنجی نام
#         if len(name) < 2 or len(name) > 30 or len(name.encode('utf-8')) > 100:
#             await message.answer("❌ نام باید بین 2 تا 30 حرف باشد. لطفاً مجدداً وارد کنید:",
#                 reply_markup=get_cancel_keyboard()  # امکان لغو وجود دارد
#             )
#             return
        
#         # بررسی کاراکترهای مجاز (با لیست کامل‌تر)
#         if not regex.match(r'^[\p{L}\s\-\.\']+$', name, re.UNICODE):
#             await message.answer("❌ نام نامعتبر است! لطفاً نام دیگری انتخاب کنید.",
#                 reply_markup=get_cancel_keyboard()  # امکان لغو وجود دارد
#             )
#             return
            
#         # بررسی کلمات ممنوعه
#         # ابتدا متن را به کلمات جداگانه تقسیم کنید
#         words_in_name = name.lower().split()

#         # سپس هر کلمه را دقیقاً با لیست کلمات ممنوعه مقایسه کنید
#         if any(word in banned_words for word in words_in_name):
#             await message.answer("❌ متن توضیحات حاوی کلمات ممنوعه است.\n 🍀لطفاً آن را اصلاح کنید.🍀",
#                 reply_markup=get_cancel_keyboard()
#             )
#             return

#         # بررسی وجود لینک در نام
#         link_patterns = [
#             r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+',
#             r'www\.[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,}',
#             r'@[a-zA-Z0-9_]+',
#             r't\.me/[a-zA-Z0-9_]+',
#             r'telegram\.me/[a-zA-Z0-9_]+'
#         ]
        
#         for pattern in link_patterns:
#             if re.search(pattern, name, re.IGNORECASE):
#                 await message.answer("❌ نام حاوی لینک است! لطفاً نام دیگری انتخاب کنید.",
#                 reply_markup=get_cancel_keyboard()  # امکان لغو وجود دارد
#             )
#                 return

#         await state.update_data(name=name)
#         await state.set_state(Form.publish_username)
        
#         # بررسی وجود یوزرنیم
#         if not message.from_user.username:
#             await state.update_data(publish_username=False)
#             await message.answer(
#                 "🍀شما یوزرنیم تلگرام ندارید. آگهی فقط با نام شما منتشر خواهد شد.🍀\n"
#                 "🍀در صورت نیاز می‌توانید ابتدا آن را برای اکانت خود تعریف کنید و پس از بازگشت به منوی اصلی، مجدداً ارسال آگهی را بزنید در غیر این صورت ادامه دهید.🍀\n\n"
#                 "📸لطفاً یک عکس یا یک ویدیوی مربوط به آگهی خود را ارسال کنید:\n"
#                 "🍀دقت کنید که شما فقط یک رسانه می‌توانید ارسال کنید و در صورت ارسال چند رسانه فقط یکی از آن‌ها توسط ربات پردازش خواهد شد.🍀"
#                 "🍀لطفاً در این بخش از نوشتن کپشن خودداری کنید. در مراحل بعد، توضیحات شما دریافت خواهد شد.🍀"
#                 "🍀حجم فایل شما باید کمتر از 50 مگابایت باشد.🍀",
#                 reply_markup=get_cancel_keyboard()
#             )
#             await state.set_state(Form.media)
#             return
        
#         keyboard = InlineKeyboardMarkup(inline_keyboard=[
#             [InlineKeyboardButton(text="✅ بله", callback_data="publish_username_yes"),
#              InlineKeyboardButton(text="❌ خیر", callback_data="publish_username_no")]
#         ])
        
#         await message.answer(
#             f"👤 آیا مایلید یوزرنیم تلگرام شما همراه آگهی منتشر شود؟",
#             reply_markup=keyboard
#         )
#     except Exception as e:
#         logger.error(f"Error in get_name: {e}")
#         await state.clear()
#         if user_id in admins:
#             await message.answer(
#                 "⚠️ خطایی رخ داد. لطفاً مجدداً از ابتدا شروع کنید.",
#                 reply_markup=await get_admin_keyboard()
#             )
#         else:
#             await message.answer(
#                 "⚠️ خطایی رخ داد. لطفاً مجدداً از ابتدا شروع کنید.",
#                 reply_markup=get_main_keyboard()
#             )

async def request_media_with_cancel(chat_id: int):
    """ارسال پیام درخواست مدیا با دکمه بازگشت و گزینه رد شدن از رسانه"""
    keyboard = ReplyKeyboardMarkup(
        keyboard=[
            [KeyboardButton(text="⏭️ این آگهی عکس یا ویدیو ندارد")],
            [KeyboardButton(text="🔙 بازگشت به منوی اصلی")]
        ],
        resize_keyboard=True
    )
    
    await bot.send_message(
        chat_id,
        "📸 لطفاً یک عکس یا ویدیوی مربوط به آگهی خود را ارسال کنید:\n"
        "یا اگر مایلید بدون رسانه ادامه دهید، روی «⏭️ این آگهی عکس یا ویدیو ندارد» کلیک کنید.\n\n"
        "🍀 نکات مهم:\n"
        "- حجم فایل باید کمتر از 50 مگابایت باشد\n"
        "- فقط یک رسانه می‌توانید ارسال کنید\n"
        "- از نوشتن کپشن در این مرحله خودداری کنید",
        reply_markup=keyboard
    )

async def request_contact_with_cancel(chat_id: int):
    """ارسال پیام درخواست مدیا با دکمه بازگشت"""
    await bot.send_message(
        chat_id,
        "📞 لطفاً یک راه ارتباطی برای نمایش در آگهی ارائه دهید:\n\n"
        "• میتوانید یک یوزرنیم دیگر (مثال: @Example)\n"
        "• یا شماره تلفن همراه (مثال: 09123456789)\n"
        "• یا هر روش ارتباطی دیگری را وارد کنید\n\n"
        "این اطلاعات به صورت عمومی در کانال نمایش داده خواهد شد.",
        reply_markup=get_cancel_keyboard()
    )


@dp.callback_query(lambda c: c.data.startswith('publish_username_'))
async def process_publish_username(callback: types.CallbackQuery, state: FSMContext):
    """پردازش انتخاب انتشار یوزرنیم"""
    try:
        await callback.answer()
        publish = callback.data == "publish_username_yes"
        await state.update_data(publish_username=publish)
        
        if publish:
            await state.set_state(Form.media)
            await callback.message.delete()
            await request_media_with_cancel(callback.message.chat.id)
        else:
            await state.set_state(Form.contact_info)
            await callback.message.delete()
            await request_contact_with_cancel(callback.message.chat.id)
            
    except Exception as e:
        logger.error(f"Error in process_publish_username: {e}")
        await callback.message.answer("⚠️ خطایی رخ داد. لطفاً مجدداً تلاش کنید.")



# تغییر handler دریافت رسانه
# @dp.message(Form.media, F.photo | F.video)  # فقط فایل‌های عکس/ویدیو
@dp.message(Form.media)
async def get_media(message: types.Message, state: FSMContext):
    """دریافت رسانه از کاربر"""
    try:
        if message.text == "🔙 بازگشت به منوی اصلی":
            await cancel_process(message, state)
            return
        
        if message.text == "⏭️ این آگهی عکس یا ویدیو ندارد":
            await state.update_data({
                'media_type': None,
                'media_id': None,
                'username': message.from_user.username
            })
            await state.set_state(Form.description)
            await message.answer(
                "📝 لطفاً توضیحی درباره این آگهی بنویسید:",
                reply_markup=get_cancel_keyboard()
            )
            return

        # بررسی نوع محتوای ارسالی
        # if not (message.photo or message.video):
        #     await message.answer(
        #         "❌ لطفاً فقط عکس یا ویدیو ارسال کنید، یا برای رد شدن از این مرحله روی «⏭️ این آگهی عکس یا ویدیو ندارد» کلیک کنید.",
        #         reply_markup=get_cancel_keyboard()
        #     )
        #     return


        # بررسی نوع محتوای ارسالی
        if not (message.photo or message.video):
            # ایجاد کیبورد با گزینه رد شدن از رسانه
            keyboard = ReplyKeyboardMarkup(
                keyboard=[
                    [KeyboardButton(text="⏭️ این آگهی عکس یا ویدیو ندارد")],
                    [KeyboardButton(text="🔙 بازگشت به منوی اصلی")]
                ],
                resize_keyboard=True
            )
            
            await message.answer(
                "❌ لطفاً فقط عکس یا ویدیو ارسال کنید، یا برای رد شدن از این مرحله روی «⏭️ این آگهی عکس یا ویدیو ندارد» کلیک کنید.",
                reply_markup=keyboard  # اضافه کردن کیبورد به پیام خطا
            )
            return




        admins = get_current_admins()

        current_state = await state.get_state()
        if current_state != Form.media.state:  # اگر کاربر در state درستی نبود
            await message.answer("⚠️ لطفاً ابتدا از منوی اصلی شروع کنید!")
            await state.clear()
            return
            
        # بررسی حجم فایل
        file_size = 0
        if message.photo:
            file_size = message.photo[-1].file_size
            media_type = 'photo'
            media_id = message.photo[-1].file_id
        elif message.video:
            file_size = message.video.file_size
            media_type = 'video'
            media_id = message.video.file_id

        if file_size > MAX_FILE_SIZE:
            await message.answer(
                "❌ حجم فایل ارسالی باید کمتر از 50 مگابایت باشد!\n"
                "🍀لطفاً فایل کوچک‌تری ارسال کنید.🍀",
                reply_markup=get_cancel_keyboard()
            )
            return  # جلوگیری از ادامه پردازش

        current_state = await state.get_state()
        if current_state != Form.media.state:
            return
            
        user_id = message.from_user.id
            
        media_type = None
        media_id = None
        
        if message.photo:
            media_type = 'photo'
            media_id = message.photo[-1].file_id
        elif message.video:
            media_type = 'video'
            media_id = message.video.file_id
        
        await state.update_data({
            'media_type': media_type, 
            'media_id': media_id,
            'username': message.from_user.username
        })
        await state.set_state(Form.description)
        
        await message.answer(
            "📝 لطفاً توضیحی درباره این آگهی بنویسید:",
            reply_markup=get_cancel_keyboard()
        )
        
        # ثبت زمان آخرین ارسال
        last_submissions[user_id] = datetime.now()
        save_all_data()

    except Exception as e:
        logger.error(f"Error in get_media: {e}")
        if user_id in admins:
            await message.answer(
                "⚠️ خطا در پردازش رسانه. لطفاً مجدداً تلاش کنید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await message.answer(
                "⚠️ خطا در پردازش رسانه. لطفاً مجدداً تلاش کنید.",
                reply_markup=get_main_keyboard()
            )
        await state.clear()


@dp.message(Form.description)
async def get_description(message: types.Message, state: FSMContext):
    try:
        if message.text == "🔙 بازگشت به منوی اصلی":
            await cancel_process(message, state)
            return
        admins = get_current_admins()

        # بررسی اینکه state هنوز معتبر است
        current_state = await state.get_state()
        if not current_state or current_state != Form.description.state:
            await state.clear()
            if user_id in admins:
                await message.answer(
                    "⚠️ زمان ورود توضیحات به پایان رسیده. لطفاً از ابتدا شروع کنید.",
                    reply_markup=await get_admin_keyboard()
                )
            else:
                await message.answer(
                    "⚠️ زمان ورود توضیحات به پایان رسیده. لطفاً از ابتدا شروع کنید.",
                    reply_markup=get_main_keyboard()
                )
            return
        
        user_id = message.from_user.id
        description = message.text
        

        word_count = len(description.split())
        if len(description) < 5 or word_count < 1:
            await message.answer(
                "❌ توضیحات شما باید بیش‌تر از یک کلمه باشد! \n 》لطفاً بیشتر توضیح دهید.",
                reply_markup=get_cancel_keyboard())
            return

        if len(description) > 600 or word_count > 100:
            await message.answer(
                "❌ توضیحات شما خیلی طولانی است!\n 》لطفاً خلاصه‌تر بنویسید.",
                reply_markup=get_cancel_keyboard())
            return
            
        # بررسی کلمات ممنوعه
        # ابتدا متن را به کلمات جداگانه تقسیم کنید
        words_in_description = description.lower().split()

        # سپس هر کلمه را دقیقاً با لیست کلمات ممنوعه مقایسه کنید
        if any(word in banned_words for word in words_in_description):
            await message.answer("❌ متن توضیحات حاوی کلمات ممنوعه است.\n 🍀لطفاً آن را اصلاح کنید.🍀",
                reply_markup=get_cancel_keyboard()
            )
            return

        # بررسی وجود لینک‌ها
        link_patterns = [
            r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+',
            r'www\.[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,}',
            r'@[a-zA-Z0-9_]+',
            r't\.me/[a-zA-Z0-9_]+',
            r'telegram\.me/[a-zA-Z0-9_]+'
        ]
        for pattern in link_patterns:
            if re.search(pattern, description, re.IGNORECASE):
                await message.answer("❌ استفاده از لینک در توضیحات مجاز نیست!",
                    reply_markup=get_cancel_keyboard()
                )
                return
            
        # ذخیره توضیحات و اطلاعات دیگر
        data = await state.get_data()
        
        # ایجاد کد پیگیری
        now = datetime.now()
        tracking_code = (
            f"{str(user_id)[-2:]}"  # دو رقم انتهای آیدی کاربر
            f"{str(now.year)[-2:]}"          # سال
            f"{now.month:02d}"     # ماه با دو رقم
            f"{now.day:02d}"       # روز با دو رقم
            f"{now.hour:02d}"      # ساعت با دو رقم
            f"{now.minute:02d}"    # دقیقه با دو رقم
            f"{now.second:02d}"    # ثانیه با دو رقم
        )


        # ذخیره اطلاعات کاربر
        user_data = {
            'Kind': data.get('Kind'),  # اضافه کردن شهر
            'category': data.get('category'),  # اضافه کردن دسته‌بندی
            'media_type': data.get('media_type'),
            'media_id': data.get('media_id'),
            'publish_username': data.get('publish_username', False),
            'username': message.from_user.username,
            'contact_info': data.get('contact_info', ''),
            'description': description,
            'submission_time': datetime.now().isoformat(),
            'status': 'pending',
            'tracking_code': tracking_code
        }
        
        # ذخیره در user_data_store
        if user_id not in user_data_store:
            user_data_store[user_id] = []
            
        if isinstance(user_data_store[user_id], dict):
            user_data_store[user_id] = [user_data_store[user_id]]

        user_data_store[user_id].append(user_data)
        
        # ثبت زمان آخرین ارسال
        last_submissions[user_id] = datetime.now()

        # بررسی و اعطای پاداش روزانه در صورت نیاز
        # reward_granted = grant_daily_reward(user_id)
        # reward_message = ""
        # if reward_granted:
        #     reward_message = f"🎉 شما با ارسال {DAILY_SUBMISSION_LIMIT} آگهی امروز، {DAILY_SUBMISSION_REWARD} امتیاز پاداش روزانه دریافت کردید."
        
        # ارسال به ادمین
        await send_to_admin(user_data, message.from_user)
        
        if message.from_user.id in admins:
            await message.answer(
                "📤 آگهی شما دریافت و برای بررسی به ادمین ارسال شد.\n\n"
                f"🔢 کد آگهی: (برای کپی روی کد کلیک کنید)\n<code>{tracking_code}</code>\n\n"
                "🕐در اسرع وقت ادمین آگهی شما را بررسی خواهد کرد.\n"
                "🔔نتیجه به شما اطلاع‌رسانی خواهد شد.\n",
                # f"{reward_message}",

                parse_mode="HTML",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await message.answer(
                "📤 آگهی شما دریافت شد و برای بررسی به ادمین ارسال شد.\n\n"
                f"🔢 کد آگهی شما: (برای کپی روی کد کلیک کنید)\n<code>{tracking_code}</code>\n\n"
                "🍀در اسرع وقت ادمین آگهی شما را بررسی خواهد کرد.🍀\n"
                "🍀نتیجه به شما اطلاع‌رسانی خواهد شد.🍀\n",
                # f"{reward_message}",
                parse_mode="HTML",
                reply_markup= get_main_keyboard()
            )

        # پاک کردن state
        await state.clear()

        # ذخیره تغییرات
        save_all_data()

    except Exception as e:
        logger.error(f"Error in get_description: {e}", exc_info=True)
        await state.clear()
        if user_id in admins:
            await message.answer(
                "⚠️ خطا در پردازش توضیحات. لطفاً از منوی اصلی دوباره شروع کنید.",
                reply_markup=await get_admin_keyboard()
            )
        else:
            await message.answer(
                "⚠️ خطا در پردازش توضیحات. لطفاً از منوی اصلی دوباره شروع کنید.",
                reply_markup=get_main_keyboard()
            )

# تغییر تابع ارسال به ادمین
async def send_to_admin(data: dict, user: types.User):
    """ارسال رسانه به ادمین برای بررسی"""
    try:
        # ایجاد شناسه یکتا برای مدیا
        media_unique_id = f"mem_{user.id}_{datetime.now().timestamp()}"
        data['media_unique_id'] = media_unique_id
        admins = get_current_admins()
        
        # ذخیره اطلاعات کاربر
        if user.id not in user_data_store:
            user_data_store[user.id] = []  # تغییر به لیست
        
        # ایجاد ساختار داده برای آگهی جدید
        memory_data = {
            'Kind': data.get('Kind'),
            'category': data.get('category'),
            'media_unique_id': media_unique_id,
            'user_id': user.id,
            'name': user.full_name,
            'username': user.username,
            'media_type': data.get('media_type'),
            'media_id': data.get('media_id'),
            'description': data.get('description'),
            'publish_username': data.get('publish_username', False),
            'contact_info': data.get('contact_info', ''),
            'submission_time': datetime.now().isoformat(),
            'status': 'pending',
            'tracking_code': data.get('tracking_code')
        }
        
        # اضافه کردن آگهی جدید به لیست آگهی‌های کاربر
        user_data_store[user.id].append(memory_data)

        contact_display = ""
        if data.get('contact_info'):
            contact_display = f"📞 تماس: {data['contact_info']}"
        
        caption = (
            f"📌 آگهی جدید برای انتشار\n\n"
            f"🆔 آیدی کاربر: <code>{user.id}</code>\n"
            f"🔗 یوزرنیم: @{user.username}\n"
            f"{contact_display}\n\n"
            
            f"📋 نوع آگهی: {data.get('Kind', 'نامشخص')}\n"
            f"📂 دسته‌بندی: {data.get('category', 'نامشخص')}\n\n"
            f"🔢 کد آگهی: <code>{data.get('tracking_code', 'نامشخص')}</code>\n\n"
            f"📝 توضیحات کاربر:\n{data.get('description', 'بدون توضیح')}"
        )
        
        keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [
                InlineKeyboardButton(text="✅ تایید", callback_data=f"approve_{media_unique_id}"),
                InlineKeyboardButton(text="❌ رد", callback_data=f"reject_{media_unique_id}")
            ],
            [
                InlineKeyboardButton(text="⛔ بلاک", callback_data=f"block_{media_unique_id}"),
                InlineKeyboardButton(text="⚠️ اخطار", callback_data=f"warn_{media_unique_id}")
            ]
        ])
        
        # مقداردهی اولیه برای پیگیری پیام‌های ادمین‌ها
        if media_unique_id not in admin_messages_tracking:
            admin_messages_tracking[media_unique_id] = {}

        # ارسال به همه ادمین‌ها
        for admin_id in admins:
            try:
                if data.get('media_type') and data.get('media_id'):
                    if data['media_type'] == 'photo':
                        message = await bot.send_photo(
                        chat_id=admin_id,
                            photo=data['media_id'],
                            caption=caption,
                            reply_markup=keyboard,
                            parse_mode="HTML"
                        )
                    else:
                        message = await bot.send_video(
                            chat_id=admin_id,
                            video=data['media_id'],
                            caption=caption,
                            reply_markup=keyboard,
                            parse_mode="HTML"
                        )
                    
                else:
                    # ارسال بدون رسانه (فقط متن)
                    message = await bot.send_message(
                        chat_id=admin_id,
                        text=caption,
                        reply_markup=keyboard,
                        parse_mode="HTML"
                    )

                # ذخیره اطلاعات پیام برای ویرایش بعدی
                if media_unique_id not in admin_messages_tracking:
                    admin_messages_tracking[media_unique_id] = {}
                admin_messages_tracking[media_unique_id][admin_id] = message.message_id
            except Exception as e:
                logger.error(f"Error sending to admin {admin_id}: {e}")
        
        save_all_data()
    except Exception as e:
        logger.error(f"Error in send_to_admin: {e}")
        raise


async def update_admin_messages(media_unique_id: str, new_text: str, reply_markup=None):
    """به‌روزرسانی پیام‌های ادمین‌ها پس از تغییر وضعیت"""
    if media_unique_id not in admin_messages_tracking:
        return
        
    for admin_id, message_id in admin_messages_tracking[media_unique_id].items():
        try:
            # دریافت پیام فعلی برای بررسی نوع محتوا
            message = await bot.get_message(admin_id, message_id)
            
            if message.photo:
                await bot.edit_message_caption(
                    chat_id=admin_id,
                    message_id=message_id,
                    caption=new_text,
                    reply_markup=reply_markup
                )
            elif message.video:
                await bot.edit_message_caption(
                    chat_id=admin_id,
                    message_id=message_id,
                    caption=new_text,
                    reply_markup=reply_markup
                )
            else:
                await bot.edit_message_text(
                    chat_id=admin_id,
                    message_id=message_id,
                    text=new_text,
                    reply_markup=reply_markup
                    )
        except Exception as e:
            logger.error(f"Error updating message for admin {admin_id}: {e}")


# تابع جدید برای محاسبه امتیاز کاربر
def calculate_user_score(user_id: int) -> float:
    if user_id not in user_scores:
        return 0.0
    
    user_score = user_scores[user_id]
    
    # میانگین امتیازهای دریافتی
    avg_rating = user_score.total_ratings #/ user_score.rating_count if user_score.rating_count > 0 else 0
    
    # امتیاز دعوت‌ها
    invite_points = user_score.invited_users * INVITE_REWARD
    
    # امتیاز پاداش روزانه
    daily_reward_points = user_score.total_score
    
    return avg_rating + invite_points + daily_reward_points


@dp.callback_query(lambda c: c.data.startswith('rate_'))
async def handle_rating(callback: types.CallbackQuery):
    """پردازش امتیازدهی به آگهی"""
    try:

        # بررسی عضویت در کانال
        if not await is_user_member(callback.from_user.id):
            await callback.answer("🙏🏻 برای رأی دادن به آگهی‌ها باید در کانال عضو باشید!", show_alert=True)
            return
        
        parts = callback.data.split('_')
        if len(parts) != 3:
            await callback.answer("❌ فرمت داده نامعتبر!", show_alert=True)
            return
            
        rating = int(parts[1])  # امتیاز (1-5)
        media_owner_id = int(parts[2])  # صاحب آگهی
        voter_id = callback.from_user.id  # رأی دهنده

        # یافتن media_unique_id از channel_posts_store
        media_unique_id = None
        for post_id, post_data in channel_posts_store.items():
            if str(callback.message.message_id) == str(post_data.get('message_id')):
                media_unique_id = post_id
                break

        if not media_unique_id:
            logger.error(f"پست با message_id {callback.message.message_id} یافت نشد!")
            await callback.answer("❌ اطلاعات مدیا یافت نشد!", show_alert=True)
            return

        # بررسی و ایجاد رکوردهای لازم
        if media_unique_id not in rating_votes:
            rating_votes[media_unique_id] = MediaRating()
            logger.info(f"رکورد جدید برای {media_unique_id} ایجاد شد")

        if media_owner_id not in user_scores:
            user_scores[media_owner_id] = UserScore()
            logger.info(f"رکورد امتیاز جدید برای کاربر {media_owner_id} ایجاد شد")

        media_rating = rating_votes[media_unique_id]
        user_score = user_scores[media_owner_id]

        # تشخیص نوع رأی (جدید یا تغییر رأی)
        user_has_voted = str(voter_id) in media_rating.ratings
        log_action = "تغییر رأی" if user_has_voted else "رأی جدید"
        
        if user_has_voted:
            old_rating = media_rating.ratings[str(voter_id)]
            media_rating.total_score -= old_rating
            user_score.total_ratings -= old_rating
        else:
            media_rating.vote_count += 1
            user_score.rating_count += 1

        # ثبت رأی جدید
        media_rating.ratings[str(voter_id)] = rating
        media_rating.total_score += rating
        user_score.total_ratings += rating

        # محاسبه میانگین جدید
        avg_rating = media_rating.total_score / media_rating.vote_count if media_rating.vote_count > 0 else 0

        # به‌روزرسانی پیام در کانال
        original_caption = callback.message.caption or ""
        new_caption = '\n'.join(
            line for line in original_caption.split('\n') 
            if not line.startswith('🌟 امتیاز:')
        )
        new_caption += f"\n🌟 امتیاز: {avg_rating:.1f} از 5 ({media_rating.vote_count} رأی)"

        try:
            if callback.message.photo:
                await callback.message.edit_caption(
                    caption=new_caption,
                    reply_markup=callback.message.reply_markup
                )
            elif callback.message.video:
                await callback.message.edit_caption(
                    caption=new_caption,
                    reply_markup=callback.message.reply_markup
                )
        except Exception as e:
            logger.error(f"خطا در ویرایش پیام: {str(e)}")
            await callback.answer("🔥 این رأی قبلاً ثبت شده است.")
            return

        # پاسخ به کاربر
        await callback.answer(f"✅ رأی شما (⭐{rating}) ثبت شد!")
        
        # ذخیره فوری تغییرات
        save_all_data()
        logger.info(
            f"{log_action} توسط {voter_id} برای پست {media_unique_id}\n"
            f"امتیاز: {rating} | میانگین جدید: {avg_rating:.1f}"
        )

    except Exception as e:
        logger.error(f"خطا در handle_rating: {str(e)}", exc_info=True)
        await callback.answer("⚠️ خطای سیستمی در ثبت رأی!", show_alert=True)


def create_rating_keyboard(user_id: int, channel_media_id: str, current_status="برقرار است ✅") -> InlineKeyboardMarkup:
    return InlineKeyboardMarkup(inline_keyboard=[
        # [
        #     InlineKeyboardButton(text="⭐ 1", callback_data=f"rate_1_{user_id}"),
        #     InlineKeyboardButton(text="⭐ 2", callback_data=f"rate_2_{user_id}"),
        #     InlineKeyboardButton(text="⭐ 3", callback_data=f"rate_3_{user_id}"),
        #     InlineKeyboardButton(text="⭐ 4", callback_data=f"rate_4_{user_id}"),
        #     InlineKeyboardButton(text="⭐ 5", callback_data=f"rate_5_{user_id}"),
        # ],
        [
            InlineKeyboardButton(
                text="🗑️ حذف آگهی",
                callback_data=f"delete_memory_{channel_media_id}"
            ),
            InlineKeyboardButton(
                text=f"{current_status}",
                callback_data=f"toggle_status_{channel_media_id}"
            )
        ],
        [
            InlineKeyboardButton(
                text="⚠️ گزارش آگهی", 
                callback_data=f"report_{channel_media_id}"  # انتقال صحیح شناسه
            ),
            InlineKeyboardButton(
                text="💠 ارسال آگهی",
                url=f"https://t.me/{BOT_USERNAME}?start"
            )
        ]
    ])



@dp.callback_query(lambda c: c.data.startswith('report_'))
async def handle_report(callback: types.CallbackQuery):
    try:
        user_id = callback.from_user.id
        channel_media_id = callback.data[len('report_'):]
        admins = get_current_admins()
        
        # بارگذاری گزارش‌های موجود از فایل
        reports_data = load_reports_data()
        
        # بررسی آیا کاربر قبلاً این آگهی را گزارش داده (چه بررسی شده یا نشده)
        user_reports = reports_data.get(str(user_id), {})
        
        # بررسی در دو حالت:
        # 1. آیا کاربر قبلاً این آگهی خاص را گزارش داده؟
        # 2. آیا کاربر اخیراً (مثلاً در 24 ساعت گذشته) گزارش‌های زیادی ارسال کرده؟
        if channel_media_id in user_reports:
            await callback.answer("⚠️ شما قبلاً این آگهی را گزارش داده‌اید!", show_alert=True)
            return
            
        # محدودیت تعداد گزارش‌های کاربر در بازه زمانی
        report_limit = 5  # حداکثر 5 گزارش در 24 ساعت
        time_window = timedelta(hours=24)
        
        user_recent_reports = 0
        for report_data in user_reports.values():
            report_time = datetime.fromisoformat(report_data['report_time'])
            if datetime.now() - report_time < time_window:
                user_recent_reports += 1
                
        if user_recent_reports >= report_limit:
            await callback.answer(
                f"⛔ شما بیش از حد مجاز گزارش ارسال کرده‌اید!\n"
                f"لطفاً فردا مجدداً تلاش کنید.",
                show_alert=True
            )
            return
        
        # بررسی وجود آگهی در کانال
        if channel_media_id not in channel_posts_store:
            logger.error(f"آگهی با شناسه {channel_media_id} یافت نشد!")
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
        
        memory_data = channel_posts_store[channel_media_id]
        # دریافت اطلاعات کامل گزارش‌دهنده
        reporter = callback.from_user
        reporter_name = f"{reporter.first_name} {reporter.last_name}" if reporter.last_name else reporter.first_name
        reporter_username = f"@{reporter.username}" if reporter.username else "ندارد"

        # ساخت متن گزارش
        report_msg = (
            f"⚠️ **گزارش جدید برای آگهی شماره #{memory_data.get('memory_number', '?')}**\n\n"
            f"🔢 کد آگهی: `{memory_data.get('tracking_code', '?')}`\n"
            f"🗑 کد حذف آگهی: {memory_data.get('message_id')}\n\n"
            f"👤 **ارسال‌کننده:**\n"
            f"👤 نام: {memory_data.get('name', '?')}\n"
            f"🆔 آیدی: `{memory_data.get('user_id', '?')}` (@{memory_data.get('username')})\n"
            f"📅 تاریخ ارسال: {memory_data.get('timestamp', '?')}\n\n"
            f"👮 **گزارش‌دهنده:**\n"
            f"👮 نام: {reporter_name}\n"
            f"🆔 آیدی: `{reporter.id}` ({reporter_username})\n\n"
        )
        
        # ایجاد کیبورد گزارش
        report_keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [
                InlineKeyboardButton(
                    text="🔍 مشاهده آگهی", 
                    url=f"https://t.me/{CHANNEL_ID[1:]}/{memory_data.get('message_id')}"
                ),
                InlineKeyboardButton(
                    text="✅ بررسی شد", 
                    callback_data=f"delete_report_{channel_media_id}_{reporter.id}"
                )
            ]
        ])
        
        # ارسال گزارش به تمام ادمین‌ها و ذخیره message_idها
        report_id = f"{channel_media_id}_{reporter.id}"
        report_messages[report_id] = {}
        
        for admin_id in admins:
            try:
                sent_message = await bot.send_message(
                    admin_id, 
                    report_msg, 
                    parse_mode="Markdown",
                    reply_markup=report_keyboard
                )
                report_messages[report_id][admin_id] = sent_message.message_id
            except Exception as e:
                logger.error(f"خطا در ارسال گزارش به ادمین {admin_id}: {e}")
        
        save_report_messages(report_messages)  # ذخیره در فایل

        # ذخیره گزارش در فایل
        if str(user_id) not in reports_data:
            reports_data[str(user_id)] = {}

        reports_data[str(user_id)][channel_media_id] = {
            'report_time': datetime.now().isoformat(),
            'memory_number': memory_data.get('memory_number'),
            'tracking_code': memory_data.get('tracking_code'),
            'report_id': report_id
        }

        save_reports_data(reports_data)
        await callback.answer("✅ گزارش شما با موفقیت ارسال شد! \n 🍀چنانچه گزارشتان نیاز به توضیح دارد با پشتیبانی ارتباط بگیرید.🍀", show_alert=True)
    
    except Exception as e:
        logger.error(f"خطای سیستمی در handle_report: {e}", exc_info=True)
        await callback.answer("⚠️ خطای سیستمی! لطفاً به ادمین اطلاع دهید.", show_alert=True)

@dp.callback_query(lambda c: c.data.startswith('delete_report_'))
async def handle_delete_report(callback: types.CallbackQuery):
    admins = get_current_admins()

    if callback.from_user.id not in admins:
        await callback.answer("⛔ فقط ادمین‌ها می‌توانند گزارش را بررسی کنند.", show_alert=True)
        return
    
    try:
        parts = callback.data.split('_')
        channel_media_id = '_'.join(parts[2:-1])
        reporter_id = int(parts[-1])
        report_id = f"{channel_media_id}_{reporter_id}"
        admin_username = f"@{callback.from_user.username}" if callback.from_user.username else "ادمین"

        # بارگذاری گزارش‌های ذخیره شده
        reports_data = load_reports_data()
        
        # دریافت اطلاعات گزارش‌دهنده
        try:
            reporter = await bot.get_chat(reporter_id)
            reporter_name = f"{reporter.first_name} {reporter.last_name}" if reporter.last_name else reporter.first_name
            reporter_username = f"@{reporter.username}" if reporter.username else "ندارد"
        except Exception as e:
            logger.error(f"Error getting reporter info: {e}")
            reporter_name = "نامشخص"
            reporter_username = "ندارد"

        # ویرایش پیام برای تمام ادمین‌ها
        if report_id in report_messages:
            for admin_id, message_id in report_messages[report_id].items():
                try:
                    # دریافت اطلاعات آگهی
                    memory_data = channel_posts_store.get(channel_media_id, {})

                    # متن جدید برای پیام
                    new_text = (
                        f"✅ **گزارش بررسی شده آگهی #{memory_data.get('memory_number', '?')}**\n\n"
                        f"🔢 کد آگهی: `{memory_data.get('tracking_code', '?')}`\n"
                        f"👤 **ارسال‌کننده:**\n"
                        f"👤 نام: {memory_data.get('name', '?')}\n"
                        f"🆔 آیدی: `{memory_data.get('user_id', '?')}` (@{memory_data.get('username', 'ندارد')})\n"
                        f"📅 تاریخ ارسال: {memory_data.get('timestamp', '?')}\n\n"
                        f"👮 **گزارش‌دهنده:**\n"
                        f"👮 نام: {reporter_name}\n"
                        f"🆔 آیدی: `{reporter_id}` ({reporter_username})\n\n"
                        f"👮 **بررسی شده توسط ادمین:** {admin_username}\n"
                        f"🕒 زمان بررسی: {datetime.now().strftime('%Y-%m-%d %H:%M')}"
                    )


                    await bot.edit_message_text(
                        chat_id=admin_id,
                        message_id=message_id,
                        text=new_text,
                        reply_markup=None,
                        parse_mode="Markdown"
                    )
                except Exception as e:
                    logger.error(f"خطا در ویرایش پیام برای ادمین {admin_id}: {e}")
            
            # حذف از حافظه و فایل
            del report_messages[report_id]
            save_report_messages(report_messages)
        
        # به‌روزرسانی وضعیت گزارش
        if str(reporter_id) in reports_data and channel_media_id in reports_data[str(reporter_id)]:
            reports_data[str(reporter_id)][channel_media_id]['status'] = 'reviewed'
            reports_data[str(reporter_id)][channel_media_id]['review_time'] = datetime.now().isoformat()
            save_reports_data(reports_data)
        
        await callback.answer("✅ گزارش با موفقیت بررسی شد!", show_alert=True)
        
    except Exception as e:
        logger.error(f"خطای سیستمی در handle_delete_report: {e}", exc_info=True)
        await callback.answer("⚠️ خطای سیستمی در بررسی گزارش!", show_alert=True)


async def cleanup_old_reports():
    """پاکسازی گزارش‌های قدیمی از حافظه و فایل"""
    current_time = datetime.now()
    reports_data = load_reports_data()
    
    # پاکسازی از حافظه موقت - فقط گزارش‌های بررسی شده را پاک کنید
    for report_id in list(report_messages.keys()):
        channel_media_id, reporter_id = report_id.split('_')
        if str(reporter_id) in reports_data and channel_media_id in reports_data[str(reporter_id)]:
            report_data = reports_data[str(reporter_id)][channel_media_id]
            if report_data.get('status') == 'reviewed':
                del report_messages[report_id]
    
    # پاکسازی از فایل ذخیره‌سازی - فقط گزارش‌های بررسی نشده قدیمی را حذف کنید
    for user_id in list(reports_data.keys()):
        for channel_media_id in list(reports_data[user_id].keys()):
            report_data = reports_data[user_id][channel_media_id]
            report_time = datetime.fromisoformat(report_data['report_time'])
            
            # فقط گزارش‌های بررسی نشده که بیش از 30 روز از آنها گذشته را حذف کنید
            if report_data.get('status') == 'pending' and (current_time - report_time) > timedelta(days=30):
                del reports_data[user_id][channel_media_id]
        
        # اگر کاربر هیچ گزارشی ندارد، کلید آن را حذف کنید
        if not reports_data[user_id]:
            del reports_data[user_id]
    
    save_reports_data(reports_data)



# تغییر handler تایید ادمین برای پشتیبانی از ویدئو
# در تابع handle_approve (جایی که آگهی به کانال ارسال می‌شود)
@dp.callback_query(lambda c: c.data.startswith('approve_'))
async def handle_approve(callback: types.CallbackQuery):
    admins = get_current_admins()

    """پردازش تایید آگهی توسط ادمین"""
    if callback.from_user.id not in admins:
        await callback.answer("⛔ فقط ادمین‌ها می‌توانند آگهی‌ای را تأیید یا رد کنند.", show_alert=True)
        return
    
    try:
        global memory_counter
        admin_username = callback.from_user.username or "ادمین"
        
        # اصلاح شده: دریافت media_unique_id به صورت ایمن
        parts = callback.data.split('_')
        if len(parts) < 2:
            await callback.answer("❌ فرمت داده نامعتبر!", show_alert=True)
            return
            
        media_unique_id = '_'.join(parts[1:])  # تمام بخش‌های بعد از approve_ را بگیر
        
        # یافتن آگهی در بین تمام آگهی‌های کاربران
        target_memory = None
        user_id = None
        
        # جستجوی عمیق در user_data_store برای یافتن آگهی
        for uid, memories in user_data_store.items():
            if not isinstance(memories, list):  # اگر لیست نیست، تبدیل کنیم
                memories = [memories]
                
            for memory in memories:
                if isinstance(memory, dict) and memory.get('media_unique_id') == media_unique_id:
                    target_memory = memory
                    user_id = uid
                    break
            if target_memory:
                break
        
        if not target_memory:
            logger.error(f"آگهی با media_unique_id {media_unique_id} یافت نشد!")
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
        
        # افزایش شمارنده آگهی‌ها
        memory_counter += 1
        memory_number = memory_counter
        save_data_to_file(memory_counter, MEMORY_COUNTER_FILE)

        # ایجاد/به‌روزرسانی رکورد امتیاز کاربر
        if user_id not in user_scores:
            user_scores[user_id] = UserScore()
        
        # افزایش تعداد ارسال‌های کاربر
        if hasattr(user_scores[user_id], 'submission_count'):
            user_scores[user_id].submission_count += 1
        else:
            user_scores[user_id].submission_count = 1

        status_text = f"\n\n✅ وضعیت: تایید شده توسط ادمین @{admin_username} (آگهی #{memory_number})"
        
        # 1. ویرایش پیام اصلی ادمین تاییدکننده
        try:
            original_text = callback.message.caption or callback.message.text
            if callback.message.content_type == 'photo':
                await callback.message.edit_caption(
                    caption=original_text + status_text,
                    reply_markup=None
                )
            elif callback.message.content_type == 'video':
                await callback.message.edit_caption(
                    caption=original_text + status_text,
                    reply_markup=None
                )
            else:
                await callback.message.edit_text(
                    text=original_text + status_text,
                    reply_markup=None
                )
        except Exception as e:
            logger.error(f"Error editing approving admin's message: {e}")

        # 2. ویرایش پیام‌های سایر ادمین‌ها
        if media_unique_id in admin_messages_tracking:
            for admin_id, message_id in admin_messages_tracking[media_unique_id].items():
                if admin_id == callback.from_user.id:
                    continue
                    
                try:
                    message = await bot.get_message(admin_id, message_id)
                    if message.content_type == 'photo':
                        await bot.edit_message_caption(
                            chat_id=admin_id,
                            message_id=message_id,
                            caption=original_text + status_text,
                            reply_markup=None
                        )
                    elif message.content_type == 'video':
                        await bot.edit_message_caption(
                            chat_id=admin_id,
                            message_id=message_id,
                            caption=original_text + status_text,
                            reply_markup=None
                        )
                    else:
                        await bot.edit_message_text(
                            chat_id=admin_id,
                            message_id=message_id,
                            text=original_text + status_text,
                            reply_markup=None
                        )
                except Exception as e:
                    logger.error(f"Error updating message for admin {admin_id}: {e}")
                    continue
        
        user_rank = get_user_rank(user_id)
        Kind = target_memory.get('Kind', 'نامشخص')
        category = target_memory.get('category', 'نامشخص')

        # آماده‌سازی متن برای کانال
        caption = f"#آگهی_{memory_number}\n"
        # جداسازی ایموجی و متن از category
        category_parts = category.split(' ', 1)  # فقط اولین فاصله را جدا می‌کند
        emoji = category_parts[0]  # ایموجی
        category_text = category_parts[1] if len(category_parts) > 1 else category  # متن دسته‌بندی
        
        # تبدیل فاصله‌ها به آندرلاین در متن‌ها
        Kind_text = Kind.split(' ', 2)[1:]  # حذف ایموجی و گرفتن بقیه متن
        Kind_text = '_'.join(' '.join(Kind_text).replace('(', '').replace(')', '').split())  # تبدیل فاصله‌ها به آندرلاین و حذف پرانتزها
        category_text = '_'.join(category_text.split())  # تبدیل فاصله‌ها به آندرلاین
        
        # ساخت کپشن با فرمت جدید
        caption += f"{Kind.split(' ', 1)[0]}#{Kind_text} {emoji}#{category_text}\n\n"

        if target_memory.get('description'):
            caption += f"📝 {target_memory['description']}\n"

        if target_memory.get('publish_username', False) and target_memory.get('username'):
            caption += f"🔗 @{target_memory['username']}\n\n"
        elif target_memory.get('contact_info'):
            caption += f"📞 تماس: {target_memory['contact_info']}\n\n"

        caption += f"\n{CHANNEL_ID}"
        # ارسال به کانال
        try:
            if target_memory.get('media_type') and target_memory.get('media_id'):
                if target_memory['media_type'] == 'photo':
                    sent_message = await bot.send_photo(
                        chat_id=CHANNEL_ID,
                        photo=target_memory['media_id'],
                        caption=caption
                    )
                else:
                    sent_message = await bot.send_video(
                        chat_id=CHANNEL_ID,
                        video=target_memory['media_id'],
                        caption=caption
                    )
            else:
                # ارسال فقط متن به کانال
                sent_message = await bot.send_message(
                    chat_id=CHANNEL_ID,
                    text=caption
                )
            
            # ذخیره لینک آگهی برای کاربر با اطلاعات کامل
            user_memories = load_user_memories()
            if str(user_id) not in user_memories:
                user_memories[str(user_id)] = []
                
            memory_link = f"https://t.me/{CHANNEL_IDM}/{sent_message.message_id}"
            user_memories[str(user_id)].append({
                'memory_number': memory_number,
                'link': memory_link,
                'timestamp': datetime.now().isoformat(),
                'tracking_code': target_memory.get('tracking_code'),
                'message_id': sent_message.message_id,
                'media_type': target_memory.get('media_type'),
                'description': target_memory.get('description', ''),
                'status': 'active'
            })
            save_user_memories(user_memories)

            # ساخت channel_media_id با message_id واقعی
            channel_media_id = f"{CHANNEL_ID}_{sent_message.message_id}"

            # ساخت کیبورد با channel_media_id
            reply_markup = create_rating_keyboard(user_id, channel_media_id, current_status="برقرار است ✅")

            # افزودن کیبورد به پیام کانال
            await sent_message.edit_reply_markup(reply_markup=reply_markup)

            channel_posts_store[channel_media_id] = {
                'user_id': user_id,
                'message_id': sent_message.message_id,
                'original_media_id': media_unique_id,
                'media_type': target_memory.get('media_type'),
                'media_id': target_memory.get('media_id'),
                'memory_number': memory_number,
                'timestamp': datetime.now().isoformat(),
                'name': target_memory.get('name'),
                'description': target_memory.get('description'),
                'username': target_memory.get('username'),
                'publish_username': target_memory.get('publish_username'),
                'tracking_code': target_memory.get('tracking_code'),
                'user_status': 'برقرار است ✅'
            }

            save_all_data()

            # به روزرسانی وضعیت آگهی
            target_memory['status'] = 'approved'
            target_memory['channel_message_id'] = sent_message.message_id
            target_memory['channel_media_id'] = channel_media_id
            target_memory['memory_number'] = memory_number
            
            await callback.answer(f"✅ آگهی با موفقیت در کانال منتشر شد. (آگهی #{memory_number})")
            
            # اطلاع به کاربر
            try:
                await bot.send_message(
                    user_id, 
                    f"🎉 آگهی شما تأیید شد. (آگهی #{memory_number})\n"
                    f"🔢 کد آگهی: <code>{int(target_memory['tracking_code'])}</code>\n",
                    parse_mode="HTML",
                    reply_markup=InlineKeyboardMarkup(inline_keyboard=[
                        [InlineKeyboardButton(
                            text="🔍 مشاهده آگهی در کانال",
                            url=f"https://t.me/{CHANNEL_IDM}/{sent_message.message_id}"
                        )]
                    ])
                )
            except Exception as e:
                logger.error(f"Error notifying user {user_id}: {e}")
            
        except Exception as send_error:
            logger.error(f"Error sending to channel: {send_error}")
            await callback.answer("⚠️ خطا در انتشار رسانه!", show_alert=True)
            
            error_text = f"\n⚠️ خطا در ارسال به کانال: {str(send_error)}"
            try:
                if callback.message.caption:
                    await callback.message.edit_caption(caption=error_text)
                else:
                    await callback.message.edit_text(text=error_text)
            except Exception as e:
                logger.error(f"Error updating error message: {e}")

        save_all_data()
    except Exception as e:
        logger.error(f"Error in handle_approve: {e}", exc_info=True)
        await callback.answer("⚠️ خطای سیستمی رخ داد! لطفاً به ادمین اصلی گزارش دهید.", show_alert=True)

@dp.callback_query(lambda c: c.data.startswith('delete_memory_'))
async def handle_delete_memory(callback: types.CallbackQuery):
    """پردازش درخواست حذف آگهی توسط کاربر"""
    try:
        # استخراج شناسه پیام کانال
        channel_media_id = callback.data[len('delete_memory_'):]
        
        # یافتن اطلاعات آگهی
        memory_data = channel_posts_store.get(channel_media_id)
        if not memory_data:
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
            
        # بررسی اینکه کاربر فعلی همان کاربر ارسال کننده است
        if callback.from_user.id != memory_data['user_id']:
            await callback.answer("⛔ شما فقط آگهی خود را می‌توانید حذف کنید", show_alert=True)
            return


        user_id = memory_data['user_id']
        
        # بارگذاری آگهی‌های کاربر
        user_memories = load_user_memories()
        user_memory_list = user_memories.get(str(user_id), [])
        
        # یافتن و به‌روزرسانی وضعیت آگهی
        for memory in user_memory_list:
            if memory.get('message_id') == memory_data['message_id']:
                memory['status'] = 'deleted_by_user' if callback.from_user.id == user_id else 'deleted_by_admin'
                break
        else:
            # اگر آگهی در لیست کاربر پیدا نشد، آن را اضافه کنید
            user_memory_list.append({
                'memory_number': memory_data.get('memory_number'),
                'link': f"https://t.me/{CHANNEL_IDM}/{memory_data['message_id']}",
                'timestamp': memory_data['timestamp'],
                'tracking_code': memory_data.get('tracking_code'),
                'message_id': memory_data['message_id'],
                'media_type': memory_data['media_type'],
                'description': memory_data.get('description', ''),
                'status': 'deleted_by_user' if callback.from_user.id == user_id else 'deleted_by_admin'
            })
        
        user_memories[str(user_id)] = user_memory_list
        save_user_memories(user_memories)






        # حذف پیام از کانال
        try:
            await bot.delete_message(
                chat_id=CHANNEL_ID,
                message_id=memory_data['message_id']
            )
        except Exception as e:
            logger.error(f"Error deleting message from channel: {e}")
            await callback.answer("⚠️ خطا در حذف آگهی!", show_alert=True)
            return
            
        # # به‌روزرسانی وضعیت در دیتابیس
        # memory_data['status'] = 'deleted_by_user'
        channel_posts_store[channel_media_id] = memory_data
        save_all_data()
        
        # اطلاع به کاربر
        await callback.answer("✅ آگهی شما با موفقیت حذف شد.", show_alert=True)
        
    except Exception as e:
        logger.error(f"Error in handle_delete_memory: {e}")
        await callback.answer("⚠️ خطای سیستمی رخ داد!", show_alert=True)

@dp.callback_query(lambda c: c.data.startswith('toggle_status_'))
async def handle_toggle_status(callback: types.CallbackQuery):
    """تغییر وضعیت آگهی بین برقرار است و بسته شد"""
    try:
        # استخراج شناسه پیام کانال
        channel_media_id = callback.data[len('toggle_status_'):]
        
        # یافتن اطلاعات آگهی
        memory_data = channel_posts_store.get(channel_media_id)
        if not memory_data:
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
            
        # بررسی اینکه کاربر فعلی همان کاربر ارسال کننده است
        # if callback.from_user.id != memory_data['user_id']:
        if callback.from_user.id != memory_data['user_id'] and callback.from_user.id != ADMIN_IDM:
            await callback.answer("⛔ شما فقط وضعیت آگهی خود را می‌توانید تغییر دهید", show_alert=True)
            return
            
        # تعیین وضعیت جدید
        current_status = memory_data.get('user_status', 'برقرار است ✅')
        new_status = 'بسته شد ❌' if current_status == 'برقرار است ✅' else 'برقرار است ✅'
        
        # به‌روزرسانی وضعیت در دیتابیس
        memory_data['user_status'] = new_status
        channel_posts_store[channel_media_id] = memory_data
        save_all_data()
        
        # ویرایش کیبورد پیام
        keyboard = create_rating_keyboard(
            memory_data['user_id'],
            channel_media_id,
            current_status=new_status
        )
        
        try:
            if callback.message.caption:
                await callback.message.edit_reply_markup(reply_markup=keyboard)
            else:
                await callback.message.edit_reply_markup(reply_markup=keyboard)
                
            await callback.answer(f"وضعیت به '{new_status}' تغییر یافت")
        except Exception as e:
            logger.error(f"Error editing message: {e}")
            await callback.answer("⚠️ خطا در تغییر وضعیت!", show_alert=True)
            
    except Exception as e:
        logger.error(f"Error in handle_toggle_status: {e}")
        await callback.answer("⚠️ خطای سیستمی رخ داد!", show_alert=True)

@dp.callback_query(lambda c: c.data.startswith('reject_'))
async def handle_reject(callback: types.CallbackQuery):
    admins = get_current_admins()

    """پردازش رد آگهی توسط ادمین"""
    if callback.from_user.id not in admins:
        await callback.answer("⛔ فقط ادمین‌ها می‌توانند آگهی‌ای را تأیید یا رد کنند.", show_alert=True)
        return
    
    try:
        admin_username = callback.from_user.username or "ادمین"
        
        # اصلاح پردازش callback.data
        parts = callback.data.split('_')
        if len(parts) < 2:
            await callback.answer("❌ فرمت داده نامعتبر!", show_alert=True)
            return
            
        media_unique_id = '_'.join(parts[1:])  # دریافت تمام بخش‌های بعد از reject_
        
        # یافتن آگهی در بین تمام آگهی‌های کاربران
        target_memory = None
        user_id = None
        
        for uid, memories in user_data_store.items():
            if not isinstance(memories, list):
                memories = [memories]
                
            for memory in memories:
                if isinstance(memory, dict) and memory.get('media_unique_id') == media_unique_id:
                    target_memory = memory
                    user_id = uid
                    break
            if target_memory:
                break
        
        if not target_memory:
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
        
        # ویرایش پیام اصلی
        original_text = callback.message.caption or callback.message.text
        new_text = f"{original_text}\n\n❌ وضعیت: رد شده توسط ادمین @{admin_username}"
        
        try:
            if callback.message.caption:
                await callback.message.edit_caption(caption=new_text, reply_markup=None)
            else:
                await callback.message.edit_text(text=new_text, reply_markup=None)
        except Exception as e:
            logger.error(f"Error editing original message: {e}")

        # ویرایش پیام برای سایر ادمین‌ها
        if media_unique_id in admin_messages_tracking:
            for admin_id, message_id in admin_messages_tracking[media_unique_id].items():
                try:
                    if admin_id == callback.from_user.id:
                        continue
                        
                    await bot.edit_message_caption(
                        chat_id=admin_id,
                        message_id=message_id,
                        caption=new_text,
                        reply_markup=None
                    )
                except Exception as e:
                    logger.error(f"Error updating admin message {admin_id}: {e}")

        # به روزرسانی وضعیت آگهی
        target_memory['status'] = 'rejected'
        
        # اطلاع به کاربر
        try:
            await bot.send_message(
                user_id,
                f"❌ متأسفیم، آگهی شما با کد <code>{int(target_memory['tracking_code'])}</code> توسط ادمین رد شد.\n\n"
                "🍀لطفاً آگهی را اصلاح و مجدداً ارسال کنید.🍀\n"
                "🍀برای اطلاعات بیشتر می‌توانید با پشتیبانی تماس بگیرید.🍀",
                parse_mode="HTML"
            )
            await callback.answer("پیام رد برای کاربر ارسال شد.")
        except Exception as e:
            logger.error(f"Error sending rejection to user {user_id}: {e}")
            await callback.answer("⚠️ ارسال پیام رد به کاربر ناموفق بود!", show_alert=True)
        
        save_all_data()
    except Exception as e:
        logger.error(f"Error in handle_reject: {e}")
        await callback.answer("⚠️ خطای سیستمی رخ داد!", show_alert=True)


@dp.callback_query(lambda c: c.data.startswith('block_'))
async def handle_block(callback: types.CallbackQuery):
    admins = get_current_admins()

    """پردازش بلاک کاربر توسط ادمین"""
    if callback.from_user.id not in admins:
        await callback.answer("⛔ فقط ادمین‌ها می‌توانند آگهی‌ای را تأیید یا رد کنند.", show_alert=True)
        return
    
    try:
        admin_username = callback.from_user.username or "ادمین"


        
        # اصلاح پردازش callback.data برای دریافت media_unique_id
        parts = callback.data.split('_')
        if len(parts) < 2:
            await callback.answer("❌ فرمت داده نامعتبر!", show_alert=True)
            return
            
        media_unique_id = '_'.join(parts[1:])  # دریافت تمام بخش‌های بعد از block_
        
        # یافتن آگهی در بین تمام آگهی‌های کاربران
        target_memory = None
        user_id = None
        
        for uid, memories in user_data_store.items():
            if not isinstance(memories, list):
                memories = [memories]
                
            for memory in memories:
                if isinstance(memory, dict) and memory.get('media_unique_id') == media_unique_id:
                    target_memory = memory
                    user_id = uid
                    break
            if target_memory:
                break

        # بررسی اینکه کاربر ادمین نباشد
        if user_id in admins:
            await callback.answer("❌ نمی‌توانید ادمین‌ها را بلاک کنید!")
            return
        
        if not target_memory:
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
            
        # ویرایش پیام اصلی
        original_text = callback.message.caption or callback.message.text
        new_text = f"{original_text}\n\n⛔ وضعیت: کاربر بلاک شده توسط ادمین @{admin_username}"
        
        # ایجاد کیبورد جدید با دکمه آنبلاک
        unblock_keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [InlineKeyboardButton(text="🔓 آنبلاک کاربر", callback_data=f"unblock_{user_id}")]
        ])
        
        try:
            if callback.message.caption:
                await callback.message.edit_caption(
                    caption=new_text,
                    reply_markup=unblock_keyboard
                )
            else:
                await callback.message.edit_text(
                    text=new_text,
                    reply_markup=unblock_keyboard
                )
        except Exception as e:
            logger.error(f"Error editing original message: {e}")

        # ویرایش پیام برای سایر ادمین‌ها
        if media_unique_id in admin_messages_tracking:
            for admin_id, message_id in admin_messages_tracking[media_unique_id].items():
                try:
                    if admin_id == callback.from_user.id:
                        continue
                        
                    if callback.message.caption:
                        await bot.edit_message_caption(
                            chat_id=admin_id,
                            message_id=message_id,
                            caption=new_text,
                            reply_markup=None
                        )
                    else:
                        await bot.edit_message_text(
                            chat_id=admin_id,
                            message_id=message_id,
                            text=new_text,
                            reply_markup=None
                        )
                except Exception as e:
                    logger.error(f"Error updating admin message {admin_id}: {e}")

        # بلاک کردن کاربر
        blocked_users.add(user_id)
        save_blocked_users(blocked_users)
        
        # اطلاع به کاربر
        try:
            await bot.send_message(
                user_id,
                f"⛔ آگهی شما با کد <code>{int(target_memory['tracking_code'])}</code> رد شد و شما توسط ادمین بلاک شدید و نمی‌توانید از ربات استفاده کنید.\n\n"
                "🍀در صورت نیاز به توضیح، با پشتیبانی ارتباط بگیرید.🍀",
                parse_mode="HTML"
            )
            await callback.answer("✅ پیام رد و کاربر بلاک شد.")
        except Exception as e:
            logger.error(f"Error notifying blocked user {user_id}: {e}")
            await callback.answer("⚠️ ارسال پیام بلاک به کاربر ناموفق بود!", show_alert=True)
        
        save_all_data()
    except Exception as e:
        logger.error(f"Error in handle_block: {e}", exc_info=True)
        await callback.answer("⚠️ خطای سیستمی رخ داد!", show_alert=True)


@dp.callback_query(lambda c: c.data.startswith('unblock_'))
async def handle_unblock(callback: types.CallbackQuery):
    admins = get_current_admins()

    if callback.from_user.id not in admins:
        await callback.answer("⛔ فقط ادمین‌ها می‌توانند آگهی‌ای را تأیید یا رد کنند.", show_alert=True)
        return

    try:
        admin_username = callback.from_user.username
        _, user_id = callback.data.split('_')
        user_id = int(user_id)
        
        # آنبلاک کردن کاربر
        if user_id in blocked_users:
            blocked_users.remove(user_id)
            save_blocked_users(blocked_users)
            
            # ویرایش پیام
            original_text = callback.message.caption or callback.message.text
            new_text = f"{original_text}\n\n✅ وضعیت: کاربر آنبلاک شده توسط @{admin_username}"
            
            try:
                # بررسی نوع پیام (عکس یا ویدیو)
                if hasattr(callback.message, 'video'):
                    # پیام ویدیویی
                    await bot.edit_message_caption(
                        chat_id=callback.message.chat.id,
                        message_id=callback.message.message_id,
                        caption=new_text,
                        reply_markup=None
                    )
                elif hasattr(callback.message, 'photo'):
                    # پیام عکسی
                    await callback.message.edit_caption(
                        caption=new_text,
                        reply_markup=None
                    )
                else:
                    # پیام متنی معمولی
                    await callback.message.edit_text(
                        text=new_text,
                        reply_markup=None
                    )
            except Exception as e:
                logger.error(f"Error editing message: {e}")

            # اطلاع به کاربر
            try:
                await bot.send_message(
                    user_id,
                    "🎉 شما توسط ادمین آنبلاک شدید و می‌توانید از ربات استفاده کنید. \n"
                    "🍀لطفاً به یاد داشته باشید که در صورت تکرار تخلف، ممکن است دوباره بلاک شوید.🍀\n"
                    "🍀همچنین اگر اشتباهی بلاک شده‌اید، عذرخواهی ما را بپذیرید.🍀"
                )
            except Exception as e:
                logger.error(f"Error notifying user: {e}")

            await callback.answer("✅ کاربر آنبلاک شد.")
        else:
            await callback.answer("ℹ️ این کاربر از قبل آنبلاک بوده است.")

        save_all_data()

    except Exception as e:
        logger.error(f"Error in handle_unblock: {e}")
        await callback.answer("⚠️ خطای سیستمی رخ داد!", show_alert=True)

@dp.callback_query(lambda c: c.data.startswith('warn_'))
async def handle_warn(callback: types.CallbackQuery):
    admins = get_current_admins()

    """پردازش اخطار آگهی توسط ادمین"""
    if callback.from_user.id not in admins:
        await callback.answer("⛔ فقط ادمین‌ها می‌توانند آگهی‌ای را تأیید یا رد کنند.", show_alert=True)
        return
    
    try:
        admin_username = callback.from_user.username or "ادمین"
        
        # اصلاح پردازش callback.data
        parts = callback.data.split('_')
        if len(parts) < 2:
            await callback.answer("❌ فرمت داده نامعتبر!", show_alert=True)
            return
            
        media_unique_id = '_'.join(parts[1:])  # دریافت تمام بخش‌های بعد از reject_
        
        # یافتن آگهی در بین تمام آگهی‌های کاربران
        target_memory = None
        user_id = None
        
        for uid, memories in user_data_store.items():
            if not isinstance(memories, list):
                memories = [memories]
                
            for memory in memories:
                if isinstance(memory, dict) and memory.get('media_unique_id') == media_unique_id:
                    target_memory = memory
                    user_id = uid
                    break
            if target_memory:
                break
        
        if not target_memory:
            await callback.answer("❌ آگهی مورد نظر یافت نشد!", show_alert=True)
            return
        
        # ویرایش پیام اصلی
        original_text = callback.message.caption or callback.message.text
        new_text = f"{original_text}\n\n⚠️ وضعیت: اخطار داده شده توسط ادمین @{admin_username}"
        
        try:
            if callback.message.caption:
                await callback.message.edit_caption(caption=new_text, reply_markup=None)
            else:
                await callback.message.edit_text(text=new_text, reply_markup=None)
        except Exception as e:
            logger.error(f"Error editing original message: {e}")

        # ویرایش پیام برای سایر ادمین‌ها
        if media_unique_id in admin_messages_tracking:
            for admin_id, message_id in admin_messages_tracking[media_unique_id].items():
                try:
                    if admin_id == callback.from_user.id:
                        continue
                        
                    await bot.edit_message_caption(
                        chat_id=admin_id,
                        message_id=message_id,
                        caption=new_text,
                        reply_markup=None
                    )
                except Exception as e:
                    logger.error(f"Error updating admin message {admin_id}: {e}")

        # به روزرسانی وضعیت آگهی
        target_memory['status'] = 'rejected'
        
        # اطلاع به کاربر
        try:
            await bot.send_message(
                user_id,
                f"⚠️ اخطار ادمین: آگهی شما با کد <code>{int(target_memory['tracking_code'])}</code> مشکلی دارد.\n\n"
                "🔻لطفاً در ارسال آگهی‌های بعدی دقت بیشتری کنید.\n"
                "🔻در صورت تکرار، ممکن است دسترسی شما به ربات محدود شود.\n"
                "🔻لطفاً اگر فکر می‌کنید این اخطار اشتباه است، با پشتیبانی ارتباط بگیرید.\n\n"
                "🍀با تشکر از همکاری شما.🍀",
                parse_mode="HTML"
            )
            await callback.answer("پیام اخطار برای کاربر ارسال شد.")
        except Exception as e:
            logger.error(f"Error sending rejection to user {user_id}: {e}")
            await callback.answer("⚠️ ارسال پیام اخطار به کاربر ناموفق بود!", show_alert=True)
        
        save_all_data()
    except Exception as e:
        logger.error(f"Error in handle_warn: {e}")
        await callback.answer("⚠️ خطای سیستمی رخ داد!", show_alert=True)


        

async def find_top_memories():
    """یافتن بهترین آگهی‌های هفته جاری با قابلیت ریستارت ایمن"""
    try:
        week_start = get_week_start()
        
        recent_memories = []
        for media_id, memory_data in channel_posts_store.items():
            try:
                post_time = datetime.fromisoformat(memory_data['timestamp'])
                if post_time >= week_start:
                    if media_id in rating_votes:
                        rating_data = rating_votes[media_id]
                        if rating_data.vote_count > 0:
                            recent_memories.append({
                                'media_id': media_id,
                                'memory_data': memory_data,
                                'rating_data': rating_data,  # اضافه کردن این خط
                                'avg_score': rating_data.total_score / rating_data.vote_count
                            })
            except Exception as e:
                logger.error(f"Error processing memory {media_id}: {e}")
                continue
        
        if not recent_memories:
            return None, "هفته"
        
        # یافتن بالاترین امتیاز
        max_score = max(m['avg_score'] for m in recent_memories)
        return [m for m in recent_memories if m['avg_score'] == max_score], "هفته"
    
    except Exception as e:
        logger.error(f"خطا در یافتن آگهی‌ها: {e}")
        return None, "هفته"
    

async def announce_winners():
    """اعلام برندگان و ارسال به کانال و ادمین"""
    try:
        top_memories, time_description = await find_top_memories()
        # if not top_memories:
        #     await bot.send_message(
        #         ADMIN_ID,
        #         f"⏳ هیچ آگهی‌ای با رأی در {time_description} گذشته یافت نشد."
        #     )
        #     return
        
        # پیام برای ادمین
        admin_message = f"🏆 برندگان {time_description} گذشته:\n\n"

        winners_info = []
        winners_info_forAdmin = []


        for memory in top_memories:
            media_id = memory['media_id']
            memory_data = memory['memory_data']
            user_id = memory_data['user_id']
            user_data = {
                'name': memory_data.get('name'),
                'description': memory_data.get('description'),
                'username': memory_data.get('username'),
                'publish_username': memory_data.get('publish_username')
            }

            # دریافت اطلاعات رأی‌ها
            rating_data = rating_votes.get(media_id, MediaRating())
            

            # اطلاعات کاربر
            username = f"@{user_data.get('username', 'ندارد')}" if user_data.get('publish_username', False) else "خصوصی"
            user_info = (
                f"👤 کاربر: {user_data.get('name', 'ناشناس')}\n"
                f"🆔 آیدی: {user_id}\n"
                f"📌 یوزرنیم: {username}\n"
                f"⭐ امتیاز: {memory['avg_score']:.1f} از 5 ({memory['rating_data'].vote_count} رأی)\n"  # تغییر این خط
                f"🔢 شماره آگهی: #{memory_data.get('memory_number', '?')}\n"
                f"📅 تاریخ ارسال: {memory_data['timestamp']}\n"
                "--------------------------------\n"
            )
            winners_info.append(user_info)
            # اطلاعات کاربر برای ادمین
            username_forAdmin = f"@{user_data.get('username', 'ندارد')}"
            user_info_forAdmin = (
                f"👤 کاربر: {user_data.get('name', 'ناشناس')}\n"
                f"🆔 آیدی: {user_id}\n"
                f"📌 یوزرنیم: {username_forAdmin}\n"
                # f"⭐ امتیاز: {memory['avg_score']:.1f} از 5 ({memory['vote_count']} رأی)\n"
                f"⭐ امتیاز: {memory['avg_score']:.1f} از 5 ({memory['rating_data'].vote_count} رأی)\n"
                f"🔢 شماره آگهی: #{memory_data.get('memory_number', '?')}\n"
                f"📅 تاریخ ارسال: {memory_data['timestamp']}\n"
                "--------------------------------\n"
            )
            winners_info_forAdmin.append(user_info_forAdmin)


            # تعیین نام نمایشی
            display_name = user_data.get('name')
            if not display_name:
                display_name = f"@{user_data['username']}" if user_data.get('username') else "کاربر"
            
            # پیام برای کانال
            channel_caption = (
                f"🏆 بهترین آگهی {time_description} 🏆\n\n"
                f"⭐ امتیاز: {memory['avg_score']:.1f} از 5 ({rating_data.vote_count} رأی)\n"
                f"📷 توسط: {display_name}\n"
            )
            

            if user_data.get('publish_username', False) and user_data.get('username'):
                channel_caption += f"🔗 @{user_data['username']}\n"

            if user_data.get('description'):
                channel_caption += f"\n📝 {user_data['description']}\n"
            
            channel_caption += f"\n#آگهی_{time_description}"
            
            # ارسال آگهی برنده به کانال
            # try:
            #     if memory_data['media_type'] == 'photo':
            #         await bot.send_photo(
            #             chat_id=CHANNEL_ID,
            #             photo=memory_data['media_id'],
            #             caption=channel_caption
            #         )
            #     else:
            #         await bot.send_video(
            #             chat_id=CHANNEL_ID,
            #             video=memory_data['media_id'],
            #             caption=channel_caption
            #         )
            # except Exception as e:
            #     logger.error(f"Error sending winner to channel: {e}")
            #     admin_message += f"\n⚠️ خطا در ارسال بهترین آگهی به کانال: {e}\n"
            #     continue
        
            # اطلاع به کاربر برنده
            # try:
            #     winner_message = (
            #         f"🎉 تبریک! آگهی شما (#{memory_data.get('memory_number', '?')}) به عنوان یکی از بهترین آگهی‌های {time_description} انتخاب شد!\n\n"
            #         f"⭐ امتیاز شما: {memory['avg_score']:.1f} از 5 ({memory['rating_data'].vote_count} رأی)\n\n"  # تغییر این خط
            #         f"کد آگهی برنده شما: <code>{int(memory_data['tracking_code'])}</code>\n\n"
            #         f"👀 می‌توانید آن را در کانال مشاهده کنید: {CHANNEL_ID}"
            #     )
            #     await bot.send_message(user_id, winner_message, parse_mode="HTML")
            # except Exception as e:
            #     logger.error(f"Error notifying winner {user_id}: {e}")
            #     admin_message += f"\n⚠️ خطا در اطلاع به کاربر برنده {user_id}: {e}\n"

        # ارسال اطلاعات برندگان به ادمین
        # admin_message += "\n".join(winners_info_forAdmin)
        # await bot.send_message(ADMIN_ID, admin_message)
        
        logger.info(f"برندگان {time_description} اعلام شدند. تعداد برندگان: {len(top_memories)}")
        
    except Exception as e:
        logger.error(f"خطا در اعلام برندگان: {e}")
        await bot.send_message(ADMIN_ID, f"⚠️ خطا در اعلام برندگان: {e}")


async def generate_bot_statistics():
    admins = get_current_admins()

    """تولید آمار جامع از داده‌های ذخیره شده"""
    stats = {
        "timestamp": datetime.now().isoformat(),
        "users": {
            "total": 0,
            "active": 0,
            "blocked": len(blocked_users)
        },
        "submissions": {
            "total": 0,
            "approved": 0,
            "rejected": 0,
            "pending": 0
        },
        "ratings": {
            "total_votes": 0,
            "average_rating": 0,
            "rated_memories": 0
        },
        "admins": {
            "total": len(admins),
            "list": []
        },
        "top_users": [],
        "top_memories": [],
        "system": {
            "last_backup": "",
            "data_files": []
        }
    }

    try:
        # آمار کاربران
        all_user_ids = set()
        all_user_ids.update(user_data_store.keys())
        all_user_ids.update(user_scores.keys())
        all_user_ids.update(rating_votes.keys())
        stats["users"]["total"] = len(all_user_ids)
        stats["users"]["active"] = stats["users"]["total"] - stats["users"]["blocked"]

        # آمار ارسال‌ها
        stats["submissions"]["total"] = len(user_data_store)
        stats["submissions"]["approved"] = len(channel_posts_store)
        stats["submissions"]["rejected"] = sum(
            1 for data in user_data_store.values() 
            if data.get('status') == 'rejected'
        )
        stats["submissions"]["pending"] = stats["submissions"]["total"] - \
                                         stats["submissions"]["approved"] - \
                                         stats["submissions"]["rejected"]

        # آمار امتیازها
        total_score = 0
        total_votes = 0
        for media_id, rating in rating_votes.items():
            total_votes += rating.vote_count
            total_score += rating.total_score
        
        stats["ratings"]["total_votes"] = total_votes
        stats["ratings"]["rated_memories"] = len(rating_votes)
        if total_votes > 0:
            stats["ratings"]["average_rating"] = round(total_score / total_votes, 2)

        # اطلاعات ادمین‌ها
        admins_data = get_current_adminsdata()
        stats["admins"]["list"] = [
            {
                "id": admin_id,
                "username": data.get("username", "ندارد"),
                "name": f"{data.get('first_name', '')} {data.get('last_name', '')}".strip(),
                "join_date": data.get("date", "نامشخص")
            }
            for admin_id, data in admins_data.items()
        ]

        # برترین کاربران (بر اساس امتیاز)
        user_score_list = []
        for user_id, score_data in user_scores.items():
            if isinstance(score_data, dict):
                score_data = UserScore.from_dict(score_data)
            user_score_list.append((user_id, calculate_user_score(user_id)))
        
        user_score_list.sort(key=lambda x: x[1], reverse=True)
        # جمع‌آوری اطلاعات کاربران برتر
        top_users = []
        for user_id, score in user_score_list[:5]:  # 5 کاربر برتر
            try:
                # دریافت اطلاعات کاربر از تلگرام (به روزترین اطلاعات)
                user = await bot.get_chat(user_id)
                username = f"{user.username}" if user.username else "ندارد"
                top_users.append({
                    "user_id": user_id,
                    "score": score,
                    "username": username
                })
            except Exception as e:
                logger.error(f"Error getting user info for {user_id}: {e}")
                # اگر دریافت اطلاعات شکست خورد، از داده‌های ذخیره شده استفاده کنید
                username = "ندارد"
                # بررسی user_data_store
                if user_id in user_data_store:
                    username = f"@{user_data_store[user_id].get('username', 'ندارد')}"
                top_users.append({
                    "user_id": user_id,
                    "score": score,
                    "username": username
                })
        
        stats["top_users"] = top_users

        # برترین آگهی‌ها (بر اساس امتیاز)
        memory_score_list = []
        for media_id, rating in rating_votes.items():
            if rating.vote_count > 0:
                avg = rating.total_score / rating.vote_count
                memory_data = channel_posts_store.get(media_id, {})
                memory_score_list.append({
                    "memory_id": media_id,
                    "memory_number": memory_data.get("memory_number", "?"),
                    "average_rating": avg,
                    "vote_count": rating.vote_count,
                    "user_id": memory_data.get("user_id")
                })
        
        memory_score_list.sort(key=lambda x: x["average_rating"], reverse=True)
        stats["top_memories"] = memory_score_list[:5]  # 5 آگهی برتر

        # اطلاعات سیستم
        stats["system"]["last_backup"] = datetime.fromtimestamp(
            os.path.getmtime(USER_DATA_FILE)
        ).isoformat() if USER_DATA_FILE.exists() else "هرگز"
        
        data_files = []
        for file in DATA_DIR.glob("*.json"):
            data_files.append({
                "filename": file.name,
                "size": f"{os.path.getsize(file)/1024:.2f} KB",
                "last_modified": datetime.fromtimestamp(
                    os.path.getmtime(file)
                ).isoformat()
            })
        stats["system"]["data_files"] = data_files

    except Exception as e:
        logger.error(f"Error generating statistics: {e}")
        stats["error"] = str(e)

    return stats

@dp.message(Command("help_admin"))
async def help_admin_command(message: types.Message):
    admins = get_current_admins()

    """راهنمای دستورات و اختیارات ادمین‌ها"""
    if message.from_user.id not in admins:
        await message.answer("⛔ این دستور فقط برای ادمین‌ها قابل استفاده است!")
        return
    
    await message.answer(bot_texts["helpadmin_text"])


@dp.message(SupportForm.description)
async def handle_support_description(message: types.Message, state: FSMContext):
    """پردازش توضیحات مشکل از سوی کاربر"""
    try:
        data = await state.get_data()
        issue_type = data.get('issue_type')
        description = message.text
        
        # پاک کردن حالت فرم
        await state.clear()
        
        # تبدیل callback_data به متن قابل فهم
        issue_type_text = {
            "support_blocked": "🤖 ربات من را اشتباهی موقتا مسدود کرده است",
            "support_admin_blocked": "👤 ادمین من را اشتباهی بلاک کرده است",
            "support_warned": "⚠️ آخرین آگهی من اشتباهی اخطار گرفته است",
            "support_rejected": "❌ آخرین آگهی من اشتباهی رد شده است",
            "support_police": "👮‍♂️ از من کلاهبرداری شده است.",
            "support_admin_request": "👑 میخواهم ادمین یونی مارکت باشم",
            "support_feedback": "💡 پیشنهاد یا انتقادی برای ربات و کانال دارم",
            "support_post_deleted": "🔍 آگهی من از کانال پس از تایید حذف شده است",
            "support_Ads": "📊 در منوی '📊 آگهی‌های من'، آگهی خود را نمی‌بینم.",
            "support_other": "❓ سایر مسائل"
        }.get(issue_type, issue_type)
        
        # ارسال به ادمین‌ها
        admins = get_current_admins()
        if not admins:
            logger.error("No admins found to send support message")
            await message.answer("⚠️ خطایی در ارسال درخواست پشتیبانی رخ داد.")
            return
            
        # ساخت متن نام کاربر
        user_name = message.from_user.full_name
        if message.from_user.username:
            user_name = f"@{message.from_user.username}"
            
        # ساخت دکمه‌های پاسخ
        keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [
                InlineKeyboardButton(text="✅ انجام شد", callback_data=f"support_done_{message.from_user.id}"),
                InlineKeyboardButton(text="❌ رد شد", callback_data=f"support_rejected_{message.from_user.id}")
            ]
        ])
            
        # ذخیره پیام‌های ارسالی به ادمین‌ها و اطلاعات درخواست
        support_messages[message.from_user.id] = {
            'messages': [],
            'issue_type': issue_type_text,
            'description': description
        }
        for admin_id in admins:
            try:
                sent_message = await message.bot.send_message(
                    chat_id=admin_id,
                    text=f"📝 درخواست پشتیبانی جدید:\n\n"
                         f"👤 کاربر: {user_name}\n"
                         f"🆔 شناسه کاربر: {message.from_user.id}\n"
                         f"📌 نوع مشکل: {issue_type_text}\n"
                         f"\n{description}",
                    reply_markup=keyboard
                )
                support_messages[message.from_user.id]['messages'].append(sent_message)
                logger.info(f"Support message sent to admin {admin_id}")
            except Exception as e:
                logger.error(f"Error sending support message to admin {admin_id}: {e}")
        
        await message.answer("✅ درخواست پشتیبانی شما با موفقیت ثبت شد. \n 🍀ادمین‌ها در اسرع وقت به آن رسیدگی خواهند کرد.🍀 \n 🍀نتیجه از طریق ربات به شما اعلام خواهد شد.🍀")
        # ذخیره تعداد درخواست‌ها (اگر قبلاً در check_support_request_limit ذخیره نشده)
        user_id_str = str(message.from_user.id)
        today = datetime.now().date().isoformat()
        
        if user_id_str not in support_requests:
            support_requests[user_id_str] = {
                'last_date': today,
                'count': 1
            }
        elif support_requests[user_id_str]['last_date'] != today:
            support_requests[user_id_str] = {
                'last_date': today,
                'count': 1
            }
        else:
            support_requests[user_id_str]['count'] += 1
            
        save_support_requests(support_requests)
        
    except Exception as e:
        logger.error(f"Error in handle_support_description: {e}")
        await message.answer("⚠️ خطایی در ثبت درخواست پشتیبانی رخ داد.")




def check_support_request_limit(user_id: int) -> bool:
    """بررسی محدودیت درخواست‌های پشتیبانی روزانه"""
    try:
        today = datetime.now().date().isoformat()
        user_id_str = str(user_id)
        
        # اگر کاربر اولین درخواست امروز را می‌دهد
        if user_id_str not in support_requests:
            support_requests[user_id_str] = {
                'last_date': today,
                'count': 1
            }
            save_support_requests(support_requests)
            return True
        
        # اگر تاریخ آخرین درخواست کاربر امروز نیست، ریست کن
        if support_requests[user_id_str]['last_date'] != today:
            support_requests[user_id_str] = {
                'last_date': today,
                'count': 1
            }
            save_support_requests(support_requests)
            return True
        
        # اگر کاربر امروز درخواست داده و به حد مجاز رسیده
        if support_requests[user_id_str]['count'] >= 3:
            return False
        
        # افزایش تعداد درخواست‌های کاربر
        support_requests[user_id_str]['count'] += 1
        save_support_requests(support_requests)
        return True
        
    except Exception as e:
        logger.error(f"Error in check_support_request_limit: {e}")
        return True  # در صورت خطا محدودیت اعمال نشود
    



if __name__ == '__main__':
    # ثبت هندلر استارت‌آپ
    dp.startup.register(on_startup)
    
    try:
        # اجرای ربات
        dp.run_polling(bot)
    except Exception as e:
        logger.error(f"Fatal error: {e}")
        # ارسال پیام خطا به ادمین اصلی
        asyncio.run(bot.send_message(
            ADMIN_IDM,
            f"🔴 ربات با خطا متوقف شد!\n"
            f"⏰ زمان: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n"
            f"🚨 خطا: {str(e)}"
        ))